function main()
%{
    This processes L3 AIRS CO, OMI NO2 & SO2, and MODIS AOD data over a
    specified region and does stats. This is for the paper: 
    Field, R.D., J.E. Hickman, I.V. Geogdzhayev, K. Tsigaridis,
    S.E. Bauer "Changes in satellite retrievals of atmospheric composition 
    over China during the 2020 COVID-19 lockdowns" in ACP. 

    At the top are definitions for dates and regions. The regions are
    specified by a gadm.org shapefile, and groupings of region names within
    the shapefile. Currently it uses ADM1 units (e.g. provinces or states),
    but this could be modified for any shapefile. Or it could be modified to a list of
    rectangle-defined regions. 

    Then it extracts daily time series over regions for each dataset and
    writes these out to a csv file for each region. 
    There is the option to either start fresh or update existing csv files.
   
    Then it does some stats and make plots, reading in whatever csv time series files are there.  
    
    robert.field@columbia.edu
%}
    clear all;
    close all;
    
    addpath('/home/rfield1/code/');
    addpath('/home/rfield1/code/MAPFire/');
    addpath('/home/rfield1/code/MAPFire/COVID/');
    
    % specify data location and descriptive name for this analysis
    COVIDDir = '~/raid/observations/COVID/';
    analysisName = 'V3';%.AIRS850';
    
    % 1 to apply log transform to all averages - didn't go anywhere with
    % this, but it's there
    logTransform = 0;
    if (logTransform)
        analysisName = [analysisName '.LOG'];
    end
    
    % 1 to plot % differences rather than absolute differences
    percDiff = 0;
    
    % set to 1 to update existing csv files with newly-downloaded data, 0
    % to start fresh (i.e. any new region or start time.
    useExistingFiles = 1;
    
    % 1 to make plots from existing csv files, 0 to just make the csv files
    makePlotsFromExistingFiles =1;
    
    % 1 to make a map of the region
    makeRegionMap = 0;
    
    % flags to process individual datasets (haven't turned any off in
    % awhile, not sure what'll happen. Think it will mainly mess up determing where to start update).
    doAIRS = 1;
    doMODIS = 1;
    doOMINO2 = 1;
    doOMISO2 = 1;
        
    % for parallel processing
    % if don't want to use, remove all 'parfors' and the 'parpool' block
    nProc =4;

    % Period over which to make daily csv files.
	startYear = 2005;
	endYear = 2020;
    startDate = datenum(startYear,1,1);
    endDate = datenum(endYear,9,30); %now; 

	% did a bit of experimentation with 'poly2', but didn't go anywhere
	% with it
    regrModel = 'poly1'; %'poly2'

    %
    % For the stats, this is a list of years over which the background 
    % starting period and end of trend period are defined
    backgroundStartYearList = [2005:2016];
    nBackStartGroundYears = size(backgroundStartYearList,2);
    backGroundEndYear = 2019;
    trendEndYear = 2019;  %2019 or 2020
        
    plotStartYear = startYear;

    %
    % these define periods over which to make overlay plots 
    %
    nDateIncrements = 0;
    
    nDateIncrements = nDateIncrements+1;
    plotStartYearList(nDateIncrements) = startYear;
    plotEndYearList(nDateIncrements) = endYear;
    overlayStartMonthList(nDateIncrements) = 8;
    dateIncrementList(nDateIncrements) = 1;
    dateIncrementUnitList(nDateIncrements) = cellstr('month');    
    titleDateFormatList(nDateIncrements) = cellstr('yyyy-mm');

% this is to see daily SO2 from Nabro
%{
    nDateIncrements = nDateIncrements+1;
    plotStartYearList(nDateIncrements) = 2009;
    plotEndYearList(nDateIncrements) = 2011;
    overlayStartMonthList(nDateIncrements) = 8;
    dateIncrementList(nDateIncrements) = 1;
    dateIncrementUnitList(nDateIncrements) = cellstr('day');    
    titleDateFormatList(nDateIncrements) = cellstr('yyyy-mm');
%}
    
    %{
    nDateIncrements = nDateIncrements+1;
    plotStartYearList(nDateIncrements) = 2018;
    plotEndYearList(nDateIncrements) = endYear;
    overlayStartMonthList(nDateIncrements) = 8;
    dateIncrementList(nDateIncrements) = 1;
    dateIncrementUnitList(nDateIncrements) = cellstr('day');    
    titleDateFormatList(nDateIncrements) = cellstr('yyyy-mm');

    nDateIncrements = nDateIncrements+1;
    plotStartYearList(nDateIncrements) = 2018;
    plotEndYearList(nDateIncrements) = endYear;
    overlayStartMonthList(nDateIncrements) = 8;
    dateIncrementList(nDateIncrements) = 7;
    dateIncrementUnitList(nDateIncrements) = cellstr('day');    
    titleDateFormatList(nDateIncrements) = cellstr('yyyy-mm');

    nDateIncrements = nDateIncrements+1;
    plotStartYearList(nDateIncrements) = 2015;
    plotEndYearList(nDateIncrements) = endYear;
    overlayStartMonthList(nDateIncrements) = 8;
    dateIncrementList(nDateIncrements) = 7;
    dateIncrementUnitList(nDateIncrements) = cellstr('day');    
    titleDateFormatList(nDateIncrements) = cellstr('yyyy-mm');
%}
        
    fontSize = 10;
    plotSmoothFactor = 1;

    %
    %  Define region here. These use ADM1 shapefiles from www.gadm.org.
    % 
    domainName = 'Eastern China';    
    shapeFilePath = ['/lovelace/brewer/rfield1/storage/observations/shapefiles/' 'gadm36_CHN_1.shp'];

    %
    % specify lockdown period for this region
    %
    % baseline period for China: 2020-01-23 to 2020-04-08 as per https://en.wikipedia.org/wiki/2020_Hubei_lockdowns
    lockDownStartMonth = 1;
    lockDownStartDay = 23;
    lockDownEndMonth = 4;
    lockDownEndDay = 8;
    
    %{
    % reviewer 2 request
    lockDownStartMonth = 2;
    lockDownStartDay = 1;
    lockDownEndMonth = 2;
    lockDownEndDay = 28;
    %}
    
    % Bauwens 2020 approach
    %{
    lockDownStartMonth = 2;
    lockDownStartDay = 11;
    lockDownEndMonth = 3;
    lockDownEndDay = 24;   
    %}
    %
    
    % 1-week longer on either side
   %{
    lockDownStartMonth = 1;
    lockDownStartDay = 16;
    lockDownEndMonth = 4;
    lockDownEndDay = 15;    
    %}

    % 1-week shorter on either side
    %{
    lockDownStartMonth = 1;
    lockDownStartDay = 30;
    lockDownEndMonth = 4;
    lockDownEndDay = 1;    
    %}
    
    lockDownStr = ['lockDown.' datestr(datenum(startYear,lockDownStartMonth,lockDownStartDay),'mm.dd') '.' ...
                               datestr(datenum(startYear,lockDownEndMonth,lockDownEndDay),'mm.dd')];
    
    % specify domain for mapping
    mapLatLim =[19 44];
    mapLonLim = [95 125];
    
    % define regions over which time series will be created. This requires
    % 'regionNames' is a descriptive name you decide
    % 'regionZones' is a list of ADM1 names that will comprise averaging
    % areas for regionNames. this is made by inspecting ADM1 fields.
    nRegions = 0;
    
    nRegions = nRegions + 1;
    regionNames(nRegions) = cellstr('central east China');
    regionLabels(nRegions,:) = cellstr([cellstr(['central']);cellstr(['east China'])]);
    regionZones(nRegions).List = {'Shaanxi','Hubei','Anhui','Jiangsu','Shanxi','Henan','Hebei','Shandong','Beijing','Tianjin'};
    regionColors(nRegions,:) = [0 0 1];
    regionLabelCoords(nRegions,:) = [34 111];    
    
    nRegions = nRegions + 1;
    regionNames(nRegions) = cellstr('southern China'); % give it the name you want
    regionLabels(nRegions,:) = cellstr([cellstr(['southern China']);cellstr([''])]); % doing it like this makes for narrower map labels
    regionZones(nRegions).List = {'Guizhou','Guangxi','Hunan','Jiangxi','Guangdong','Fujian','Zhejiang'}; % provide the list of ADM1 regions to include
    regionColors(nRegions,:) = [1 0 0];
    regionLabelCoords(nRegions,:) = [25.5 109];

    
    plotStartDate = datenum(plotStartYear,1,1);
    
    % where to put the output
	outputDir = [COVIDDir analysisName '/' strrep(domainName,' ','') '/'  datestr(startDate,'yyyymmdd') '/'];% '.' datestr(endDate,'yyyymmdd') '/'];
    mkdir(outputDir);

    mapOutputDir = [outputDir 'MapInfo/'];
    mkdir(mapOutputDir);
    
    
    % regional SO2 thresholds from Kroktov et al.(2016,ACP) percentile
    % method applied to closest matching region to Eastern US, Eastern Europe, Eastern China, India and Middle East. 
    % See Appendix A1 of the paper
    %SO2ThreshPercile = 99.9;
    
    % Exclude SO2 values above this value, from Wang and Wang 2020 Atm Env,
    % presumably less sensitive to regional definitions than Kroktov
    SO2Thresh = 15;

    %
    % OMI L3 NO2 and SO2 info
    % OMI data are from 'https://acdisc.gesdisc.eosdis.nasa.gov/data/Aura_OMI_Level3/'
    %
	OMIDir = '/home/rfield1/raid/observations/OMI/'; % 
    OMINO2Product = 'OMNO2d';
    OMINO2Version = '003';
    OMINO2VarName = 'ColumnAmountNO2TropCloudScreened';
    OMINO2units = 'molec cm^{-2}';    

    OMISO2Product = 'OMSO2e';
    OMISO2Version = '003';
    OMISO2VarName = 'ColumnAmountSO2_PBL';
    OMISO2ScaleFactor = 1; % 
    OMISO2units = 'DU';    

    %
    % AIRS L3 info
    % AIRS data are from https://acdisc.gesdisc.eosdis.nasa.gov/data/Aqua_AIRS_Level3/
    %    
	AIRSDir = '/home/rfield1/raid/observations/AIRS/'; % 
    AIRSProduct = 'AIRS3STD.006';
    AIRSFileSuffix = '.L3.RetStd_IR001.v6.';
    AIRSFieldName = 'CO_VMR_A';
    AIRSAnalysisPresI = 6; % 850 hPa, should search for this instead
    AIRSAnalysisPres = 500; %
%    AIRSAnalysisPresI = 3; % 850 hPa, should search for this instead
%    AIRSAnalysisPres = 850; %
    AIRSCOScaleFactor = 1e9;
    AIRSCOUnits = 'ppbv';
    
    %
    %   MODIS L3 info
    %
    % Data are from 
    %'https://ladsweb.modaps.eosdis.nasa.gov/archive/allData/MYD08_D3/' for Aqua 
    %'https://ladsweb.modaps.eosdis.nasa.gov/archive/allData/MO08_D3/' for Terra 

	MODISDir = '/home/rfield1/raid/observations/MODISAOD/';

    AODTitleStrL3= [];
    AODFileStrL3 = [];

    nSatellites =0;
   
    nSatellites = nSatellites+1;
    satName(nSatellites) = cellstr('Aqua');
    satAODFileNameL3(nSatellites) = cellstr('MYD08_D3');
    AODTitleStrL3 = ['d)' AODTitleStrL3 ' ' char(satName(nSatellites)) ' (' char(satAODFileNameL3(nSatellites)) ')'];
	AODFileStrL3 = [AODFileStrL3 char(satName(nSatellites)) '.' char(satAODFileNameL3(nSatellites)) '.'];

    nSatellites = nSatellites+1;
    satName(nSatellites) = cellstr('Terra');
    satAODFileNameL3(nSatellites) = cellstr('MOD08_D3');
     
    AODTitleStrL3 = [AODTitleStrL3 ' & ' char(satName(nSatellites)) ' (' char(satAODFileNameL3(nSatellites)) ')'];
	AODFileStrL3 = [AODFileStrL3 char(satName(nSatellites)) '.' char(satAODFileNameL3(nSatellites)) '.'];

    AODTitleStrL3 = [cellstr(AODTitleStrL3);...
                    cellstr(' MODIS AOD, Land-DB, Ocean-DT')];
    AODTitleStrL3 = strrep(AODTitleStrL3,'_','\_');

    %
    % Aerosol particulars
    %
    landFieldNameAODL3 = 'Deep_Blue_Aerosol_Optical_Depth_550_Land_Mean';
    oceanFieldNameAODL3 = 'AOD_550_Dark_Target_Deep_Blue_Combined_Mean';
    landScaleFactorAODL3 = 0.0010000000474974513;
    oceanScaleFactorAODL3 = 0.0010000000474974513;        
   
    %
    % make a map of the different region groupings. put
    % this in a function someday
    %
    if (makeRegionMap)
        close all;
        % make a map of all regions
        figH = figure('visible','off','PaperUnits','inches','Units','inches');
        currPos = get(figH,'Position');

        set(figH,'Position',[currPos(1) currPos(2)  1.5*currPos(3) 2*currPos(4)]);
        set(figH,'Color',[1 1 1]);% box off doesn't work,'box','off');
        ax = gca;
        ax.XColor = [1 1 1];
        ax.YColor = [1 1 1];

        mapAx = axesm('MapProjection','eqaconic',... 'Xcolor','w','Ycolor','w',...
            'Frame', 'on','FEdgeColor','k','FLineWidth',1,'FFaceColor',[255/255 255/255 255/255],...
            'Grid', 'on','MeridianLabel','on','ParallelLabel','on','MLabelParallel','south',...
            'FontColor','k','MLineLocation',5,'PLineLocation',5,'FontSize',10,...
            'MapLatLimit',mapLatLim,...
            'MapLonLimit',mapLonLim);

        landareas = shaperead([  '~/raid/observations/shapefiles/TM_WORLD_BORDERS-0.3.shp'],'UseGeoCoords',true);
        geoshow(landareas,'FaceColor','none','EdgeColor','k');

        allADMShapes = shaperead(shapeFilePath,'UseGeoCoords',true);
        nADMShapes = size(allADMShapes,1);
        
        for currRegionI = 1:nRegions
            currRegionName = char(regionNames(currRegionI)); 
            currZoneList = regionZones(currRegionI).List;
            
            % go through each ADM shape and plot the ones in current zone
            % list
            currZoneCount = 0;
            for currADMI = 1:nADMShapes
                currADMName = char(allADMShapes(currADMI).NAME_1);
                if (find(strcmp(currZoneList,currADMName)))
                    currADMShape = allADMShapes(currADMI);
                    geoshow(currADMShape,'FaceColor',regionColors(currRegionI,:),'EdgeColor',[.5 .5 .5]);
                    currZoneCount = currZoneCount + 1;
                    currRegionShapeList(currZoneCount) = currADMShape;
                end        
            end
            textm(regionLabelCoords(currRegionI,1),regionLabelCoords(currRegionI,2),regionLabels(currRegionI,:),'FontSize',16);
            currRegionSHPName = [mapOutputDir strrep(currRegionName,' ','')];
            shapewrite(currRegionShapeList,currRegionSHPName);
        end

        currFN = [mapOutputDir 'analysisRegions.nRegions.' num2str(nRegions) '.png'];
        saveas(gcf,currFN);
    end

    %
	% write out code region grids for others to use. also put in a function
	% someday
    %    
    gridRes = 0.5;    
    currLat = [-90+gridRes/2:gridRes:90-gridRes/2]';
    currLon = [-180+gridRes/2:gridRes:180-gridRes/2]';
    [regionNames,regionZones,shapeArray,latGrid,lonGrid] = DefineGriddedShapesADM1(currLat,currLon,shapeFilePath,regionNames,regionZones);
    currFN = [mapOutputDir strrep(domainName,' ','') '.' num2str(gridRes) 'X' num2str(gridRes) '.CodeGrid.csv'];
    dlmwrite(currFN,shapeArray);
    currFN = [mapOutputDir strrep(domainName,' ','') '.' num2str(gridRes) 'X' num2str(gridRes) '.LatGrid.csv'];
    dlmwrite(currFN,latGrid);
    currFN = [mapOutputDir strrep(domainName,' ','') '.' num2str(gridRes) 'X' num2str(gridRes) '.LonGrid.csv'];
    dlmwrite(currFN,lonGrid);    
    
    gridRes = 0.25;    
    currLat = [-90+gridRes/2:gridRes:90-gridRes/2]';
    currLon = [-180+gridRes/2:gridRes:180-gridRes/2]';
    [regionNames,regionZones,shapeArray,latGrid,lonGrid] = DefineGriddedShapesADM1(currLat,currLon,shapeFilePath,regionNames,regionZones);
    currFN = [mapOutputDir strrep(domainName,' ','') '.' num2str(gridRes) 'X' num2str(gridRes) '.CodeGrid.csv'];
    dlmwrite(currFN,shapeArray);
    currFN = [mapOutputDir strrep(domainName,' ','') '.' num2str(gridRes) 'X' num2str(gridRes) '.LatGrid.csv'];
    dlmwrite(currFN,latGrid);
    currFN = [mapOutputDir strrep(domainName,' ','') '.' num2str(gridRes) 'X' num2str(gridRes) '.LonGrid.csv'];
    dlmwrite(currFN,lonGrid);    
        
    currFN = [mapOutputDir strrep(domainName,' ','') '.CodeDefs.csv'];
    fID = fopen(currFN,'w');
    fprintf(fID,'%s\n','-1,unclassified');
    for currRegionI = 1:nRegions
        fprintf(fID,'%d,%s\n',currRegionI,char(regionNames(currRegionI)));
    end
    fclose(fID);    
    
    
%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    Make time series over regions for all variables 
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}    
    % this is what each regional time series file contains 
 	regionalTableVarNames = {'YYYY','MM','DD','AIRS_CO_AVE','AIRS_CO_Q25','AIRS_CO_Q50','AIRS_CO_N',...
                                              'T_MODIS_AOD_AVE','T_MODIS_AOD_Q25','T_MODIS_AOD_Q50','T_MODIS_AOD_N',...
                                              'A_MODIS_AOD_AVE','A_MODIS_AOD_Q25','A_MODIS_AOD_Q50','A_MODIS_AOD_N',...
                                              'OMI_NO2_AVE','OMI_NO2_Q25','OMI_NO2_Q50','OMI_NO2_N',...
                                              'OMI_SO2_AVE','OMI_SO2_Q25','OMI_SO2_Q50','OMI_SO2_N'};

	% read in existing regional data to determine what needs updating
    currDate = startDate;
    if (useExistingFiles)
        for currRegionI = 1:nRegions
            currRegionName = char(regionNames(currRegionI));    
            currFN = [outputDir  num2str(currRegionI) '.' strrep(currRegionName,' ','')  '.DailyData.csv'];
            currTable = readtable(currFN);
            allOldDates = datenum(currTable{:,1},currTable{:,2},currTable{:,3});
            allOldRegionalTables(currRegionI).Table = currTable;
        end       

        startDateFromOldFile = min(allOldDates)
        endDateFromOldFile = max(allOldDates)
        newStartDate = addtodate(endDateFromOldFile,1,'day');
        currDate = newStartDate;
    end
    
    %
    % Pre-define date lists, needed for parallelization
    % 
    nL3Dates = 1;
    while (currDate <=endDate)
        allL3DateList(nL3Dates) = currDate;
        datestr(currDate,'yyyymmdd HH:MM:SS');
        currDate = addtodate(currDate,1,'day');
        
        % this to round off any milliseconds etc
        currDateVec = datevec(currDate);
        currDate = datenum(currDateVec(1),currDateVec(2),currDateVec(3),currDateVec(4),currDateVec(5),0);
        nL3Dates = nL3Dates + 1;
    end
    allL3DateList = allL3DateList';
    nL3Dates = size(allL3DateList,1);    
    
    % turn on parallelization
    oldPoolObj = gcp('nocreate');
    if (~isempty(oldPoolObj))
        delete(oldPoolObj);
    end
    parpool(nProc)
    pctRunOnAll warning('off','all');

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    OMI NO2
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}       
    % pre-allocate output vectors
    allOMINO2Ave = NaN*ones(nRegions,nL3Dates);
    allOMINO2Q25 = NaN*ones(nRegions,nL3Dates);
    allOMINO2Q50 = NaN*ones(nRegions,nL3Dates);
    allOMINO2Count = NaN*ones(nRegions,nL3Dates,1);    
    if (doOMINO2)
        % don't like this hardcoding, but
        % "h5read(currOMIPath,['/HDFEOS/GRIDS/ColumnAmountNO2/lat'])"
        % didn't work
        currOMILat = [-89.875:.25:89.875]';
        currOMILon = [-179.875:.25:179.875]';
        [regionNames,regionZones,shapeArray,latGrid,lonGrid] = DefineGriddedShapesADM1(currOMILat,currOMILon,shapeFilePath,regionNames,regionZones);
        nRegions = size(regionNames,2);
        tic
        parfor currDateI = 1:nL3Dates
            currDate = allL3DateList(currDateI);
            datestr(currDate)
            % get L3 AOD data
            % e.g. /home/rfield1/raid/observations/OMI/OMNO2d.003/2020/OMI-Aura_L3-OMNO2d_2020m0408_v003.he5
            currOMIPath = ['OMI-Aura_L3-' OMINO2Product '_' datestr(currDate,'yyyy') 'm' datestr(currDate,'mm') datestr(currDate,'dd') '_v' OMINO2Version '.he5'];
            currOMIPath = [OMIDir OMINO2Product '.' OMINO2Version '/' datestr(currDate,'yyyy') '/' currOMIPath];

            if (exist(currOMIPath,'file'))
                %h5disp(currOMIPath); % if want file into 
                currOMINO2 = h5read(currOMIPath,['/HDFEOS/GRIDS/ColumnAmountNO2/Data Fields/' OMINO2VarName]);
                currOMINO2Weight = h5read(currOMIPath,['/HDFEOS/GRIDS/ColumnAmountNO2/Data Fields/' 'Weight']);
                currOMINO2(find(currOMINO2<0)) = NaN;
                currOMILat = [-89.875:.25:89.875]'; % can't remember why this is being redefined.
                currOMILon = [-179.875:.25:179.875]';
                
                % find the good values at each grid point in the region,
                % and take stats
                for currRegionI = 1:nRegions
                    currGridI = find(shapeArray==currRegionI);
                    currAvg = currOMINO2(currGridI);
                    currW = currOMINO2Weight(currGridI);
                    goodI = find(~isnan(currAvg) & ~isnan(currW));
                    if (~isempty(goodI))
                        allOMINO2Ave(currRegionI,currDateI) = wmean(currAvg(goodI),currW(goodI)); %nanmean(currAvg(goodI));
                        allOMINO2Q25(currRegionI,currDateI) = prctile(currAvg(goodI),25);
                        allOMINO2Q50(currRegionI,currDateI) = prctile(currAvg(goodI),50);
                        allOMINO2Count(currRegionI,currDateI) = size(goodI,1);
                    end
                end                
            end
        end
        toc
    end

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    OMI SO2
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}       
    % pre-allocate output vectors
    
    allOMISO2Ave = NaN*ones(nRegions,nL3Dates);
    allOMISO2Q25 = NaN*ones(nRegions,nL3Dates);
    allOMISO2Q50 = NaN*ones(nRegions,nL3Dates);
    allOMISO2Count = zeros(nRegions,nL3Dates,1);
        
    if (doOMISO2)
        currOMIPath = ['OMI-Aura_L3-' OMISO2Product '_' datestr(allL3DateList(1),'yyyy') 'm' datestr(allL3DateList(1),'mm') datestr(allL3DateList(1),'dd') '_v' OMINO2Version '.he5'];
        currOMIPath = [OMIDir OMISO2Product '.' OMISO2Version '/' datestr(allL3DateList(1),'yyyy') '/' currOMIPath];
        if (exist(currOMIPath,'file'))
            currOMILat = h5read(currOMIPath,['/HDFEOS/GRIDS/OMI Total Column Amount SO2/Data Fields/Latitude']);  
            currOMILat = currOMILat(1,:);
            currOMILon = h5read(currOMIPath,['/HDFEOS/GRIDS/OMI Total Column Amount SO2/Data Fields/Longitude']);  
            currOMILon = currOMILon(:,1);        

            [regionNames,regionZones,shapeArray,latGrid,lonGrid] = DefineGriddedShapesADM1(currOMILat',currOMILon',shapeFilePath,regionNames,regionZones);
            nRegions = size(regionNames,2);

            tic
            parfor currDateI = 1:nL3Dates
                currDate = allL3DateList(currDateI)
                datestr(currDate);
                currOMIPath = ['OMI-Aura_L3-' OMISO2Product '_' datestr(currDate,'yyyy') 'm' datestr(currDate,'mm') datestr(currDate,'dd') '_v' OMINO2Version '.he5'];
                currOMIPath = [OMIDir OMISO2Product '.' OMISO2Version '/' datestr(currDate,'yyyy') '/' currOMIPath];

                if (exist(currOMIPath,'file'))
                    %h5disp(currOMIPath);
                    currOMISO2 = h5read(currOMIPath,['/HDFEOS/GRIDS/OMI Total Column Amount SO2/Data Fields/' OMISO2VarName]);
                    currOMISO2(find(currOMISO2<-10)) = NaN; % minimum value in format guide

                    currOMILat = h5read(currOMIPath,['/HDFEOS/GRIDS/OMI Total Column Amount SO2/Data Fields/Latitude']); 
                    currOMILat = currOMILat(1,:);
                    currOMILon = h5read(currOMIPath,['/HDFEOS/GRIDS/OMI Total Column Amount SO2/Data Fields/Longitude']);  
                    currOMILon = currOMILon(:,1);
                    
                    % find the good values at each grid point in the region,
                    % and take stats
                    for currRegionI = 1:nRegions
                        currGridI = find(shapeArray==currRegionI);
                        currAvg = currOMISO2(currGridI);
                        goodI = find(~isnan(currAvg));

                        if (~isempty(goodI))
                            currAvg = currAvg(goodI);
                            goodI = find(currAvg < SO2Thresh);

                            if (~isempty(goodI))
                                allOMISO2Ave(currRegionI,currDateI) = nanmean(currAvg(goodI));
                                allOMISO2Q25(currRegionI,currDateI) = prctile(currAvg(goodI),25);
                                allOMISO2Q50(currRegionI,currDateI) = prctile(currAvg(goodI),25);
                                allOMISO2Count(currRegionI,currDateI) = size(goodI,1);
                            else
                                allOMISO2Count(currRegionI,currDateI) = NaN;
                            end
                        end
                    end                
                end
            end
            toc
        end
    end    
    
%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    AIRS CO
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}       
    % pre-allocate output vectors
        
 	% get AIRS CO
	allAIRSCOL3Ave = NaN*ones(nRegions,nL3Dates);
	allAIRSCOL3Q25 = NaN*ones(nRegions,nL3Dates);
	allAIRSCOL3Q50 = NaN*ones(nRegions,nL3Dates);
	allAIRSCOL3Count = NaN*ones(nRegions,nL3Dates);

    if (doAIRS)
        
        currAIRSPath = [AIRSDir '/' AIRSProduct '/' datestr(allL3DateList(1),'yyyy') '/'  'AIRS.' datestr(allL3DateList(1),'yyyy.mm.dd') AIRSFileSuffix ];
        dirResult = dir([currAIRSPath '*.hdf']);
        currAIRSPath = [AIRSDir '/' AIRSProduct '/' datestr(allL3DateList(1),'yyyy') '/' dirResult.name]; 
        if (exist(currAIRSPath,'file') & strcmp(currAIRSPath(end-2:end),'hdf'))
            currAIRSLat = hdfread(currAIRSPath,'Latitude');
            currAIRSLon = hdfread(currAIRSPath,'Longitude');        

            [regionNames,regionZones,shapeArray,latGrid,lonGrid] = DefineGriddedShapesADM1(currAIRSLat(:,1),currAIRSLon(1,:)',shapeFilePath,regionNames,regionZones);
            nRegions = size(regionNames,2);           

            tic
            parfor currDateI = 1:nL3Dates
                currDate = allL3DateList(currDateI);

                currAIRSPath = [AIRSDir '/' AIRSProduct '/' datestr(currDate,'yyyy') '/'  'AIRS.' datestr(currDate,'yyyy.mm.dd') AIRSFileSuffix ];
                dirResult = dir([currAIRSPath '*.hdf']);
                currAIRSPath = [AIRSDir '/' AIRSProduct '/' datestr(currDate,'yyyy') '/' dirResult.name];

                if (exist(currAIRSPath,'file') & strcmp(currAIRSPath(end-2:end),'hdf'))

                    currAIRSCO = hdfread(currAIRSPath,AIRSFieldName);
                    currAIRSCO(find(currAIRSCO==-9999)) = NaN;
                    currAIRSLat = hdfread(currAIRSPath,'Latitude');
                    currAIRSLon = hdfread(currAIRSPath,'Longitude');
                    currAIRSPres = hdfread(currAIRSPath,'StdPressureLev:ascending');

                    currAIRSCO = squeeze(currAIRSCO(AIRSAnalysisPresI,:,:));
                    currAIRSCO = currAIRSCO';

                    for currRegionI = 1:nRegions
                        currGridI = find(shapeArray==currRegionI);

                        currAIRSPoints = currAIRSCO(currGridI);
                        goodI = find(~isnan(currAIRSPoints));
                        nGood = size(goodI,1);
                        currAIRSPoints = currAIRSPoints(goodI);
                        currAvg = AIRSCOScaleFactor*nanmean(currAIRSPoints(:));
                        allAIRSCOL3Ave(currRegionI,currDateI) = currAvg;
                        
                        currQ25 = AIRSCOScaleFactor*prctile(currAIRSPoints(:),25);
                        allAIRSCOL3Q25(currRegionI,currDateI) = currQ25;
                        
                        currQ50 = AIRSCOScaleFactor*prctile(currAIRSPoints(:),50);
                        allAIRSCOL3Q50(currRegionI,currDateI) = currQ50;
                        
                        allAIRSCOL3Count(currRegionI,currDateI) = nGood;
                    end
                end   
            end  
            toc
        end
    end
 

 %{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    MODIS AOD
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}       
    % pre-allocate output vectors
   
    allMODISAODL3Ave = NaN*ones(nRegions,nL3Dates,nSatellites);
    allMODISAODL3Q25 = NaN*ones(nRegions,nL3Dates,nSatellites);
    allMODISAODL3Q50 = NaN*ones(nRegions,nL3Dates,nSatellites);
    allMODISnGoodL3 =  NaN*ones(nRegions,nL3Dates,nSatellites);

    if (doMODIS)
        for currSatI = 1:nSatellites
            currSatName = char(satName(currSatI));
            currMODISAODName = char(satAODFileNameL3(currSatI));
            currMODISPathAOD = [MODISDir currMODISAODName '/' datestr(allL3DateList(1),'yyyy') '/' currMODISAODName '.' datestr(allL3DateList(1),'yyyy.mm.dd') '.hdf' ];
            if (exist(currMODISPathAOD,'file'))
                [currMODISLat,currMODISLon,currMODISAODPixels] = getMODISL3Field(currMODISPathAOD,landFieldNameAODL3,oceanFieldNameAODL3,landScaleFactorAODL3,oceanScaleFactorAODL3);

                [regionNames,regionZones,shapeArray,latGrid,lonGrid] = DefineGriddedShapesADM1(currMODISLat',currMODISLon',shapeFilePath,regionNames,regionZones);
                nRegions = size(regionNames,2);           

                tic
                parfor currDateI = 1:nL3Dates
                    currDate = allL3DateList(currDateI);

                    % get L3 AOD data
                    currMODISPathAOD = [MODISDir currMODISAODName '/' datestr(currDate,'yyyy') '/' currMODISAODName '.' datestr(currDate,'yyyy.mm.dd') '.hdf' ];
                    if (exist(currMODISPathAOD,'file'))

                        [currMODISLat,currMODISLon,currMODISAODPixels] = getMODISL3Field(currMODISPathAOD,landFieldNameAODL3,oceanFieldNameAODL3,landScaleFactorAODL3,oceanScaleFactorAODL3);
                        currMODISAODPixels = currMODISAODPixels';
                        for currRegionI = 1:nRegions
                            currGridI = find(shapeArray==currRegionI);

                            if (~isempty(currGridI))
                                currMODISPoints = currMODISAODPixels(currGridI);
                                goodI = find(~isnan(currMODISPoints));
                                allMODISnGoodL3(currRegionI,currDateI,currSatI) = size(goodI,1);
                                currAvg = nanmean(currMODISPoints(goodI));
                                allMODISAODL3Ave(currRegionI,currDateI,currSatI) = currAvg;
                                
                                currQ25 = prctile(currMODISPoints(goodI),25);
                                allMODISAODL3Q25(currRegionI,currDateI,currSatI) = currQ25;                                
                                currQ50 = prctile(currMODISPoints(goodI),50);
                                allMODISAODL3Q50(currRegionI,currDateI,currSatI) = currQ50;                                
                            end 
                        end
                    end   
                end
                toc
            end
        end
    end
     
%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Append new to old data and write out regional time series, 
but only up until last date when all data are available.
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}       

    MODISTCount = squeeze(allMODISnGoodL3(:,:,1));
    MODISACount = squeeze(allMODISnGoodL3(:,:,2));
    goodDayMatrix = [sum(allAIRSCOL3Count); sum(allOMINO2Count); sum(allOMISO2Count); sum(MODISTCount); sum(MODISACount)];
    lastGoodDateI = 0;
    currDateI = 1;
    while (currDateI <=nL3Dates)
        missingI = find(isnan(goodDayMatrix(:,currDateI)));
        if (isempty(missingI))
            lastGoodDateI = currDateI;
        end
        currDateI = currDateI+1;
    end
    
    % write out updated tables
    for currRegionI = 1:nRegions
        currRegionName = char(regionNames(currRegionI)); 

        % write out daily data
        outputTable = [allAIRSCOL3Ave(currRegionI,:)' allAIRSCOL3Q25(currRegionI,:)' allAIRSCOL3Q50(currRegionI,:)' allAIRSCOL3Count(currRegionI,:)' ...
                       squeeze(allMODISAODL3Ave(currRegionI,:,1))'  squeeze(allMODISAODL3Q25(currRegionI,:,1))'   squeeze(allMODISAODL3Q50(currRegionI,:,1))' squeeze(allMODISnGoodL3(currRegionI,:,1))' ...
                       squeeze(allMODISAODL3Ave(currRegionI,:,2))'  squeeze(allMODISAODL3Q25(currRegionI,:,2))' squeeze(allMODISAODL3Q50(currRegionI,:,2))'  squeeze(allMODISnGoodL3(currRegionI,:,2))' ... 
                       allOMINO2Ave(currRegionI,:)'  allOMINO2Q25(currRegionI,:)'  allOMINO2Q50(currRegionI,:)'  allOMINO2Count(currRegionI,:)' ...
                       allOMISO2Ave(currRegionI,:)'  allOMISO2Q25(currRegionI,:)' allOMISO2Q50(currRegionI,:)' allOMISO2Count(currRegionI,:)'];
                   
        allDateVec = datevec(allL3DateList);
        allDateVec = allDateVec(:,1:3);
        outputTable = num2cell([allDateVec outputTable]);
        
        % kinda awkward logic
        if (useExistingFiles) 
            currOldTable = num2cell(allOldRegionalTables(currRegionI).Table{:,:});
            if (lastGoodDateI)
                outputTable = outputTable(1:lastGoodDateI,:);
                outputTable = [regionalTableVarNames; currOldTable; outputTable];
            else
                outputTable = [regionalTableVarNames; currOldTable];
            end
        else
            outputTable = outputTable(1:lastGoodDateI,:);
            outputTable = [regionalTableVarNames; outputTable];
        end
        outputTable = table(outputTable);
        currFN = [outputDir  num2str(currRegionI) '.' strrep(currRegionName,' ','')  '.DailyData.csv'];
        writetable(outputTable,currFN,'WriteVariableNames',false);
    end

%{
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Do plots and stats. 
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%}     

    if (makePlotsFromExistingFiles)

        % each species has mean,q25,q50,N
        COStartCol =4;
        AODTStartCol = 8;
        AODAStartCol = 12;
        NO2StartCol = 16;
        SO2StartCol = 20;
        
        statNameList = {'mean','Q25','median'};
        statOffSetList = [0,1,2];
        nStats = size(statNameList,2);
        
        for currStatI = 1:nStats
            statName = char(statNameList(currStatI)); 
            statOffSet = statOffSetList(currStatI);

            for currRegionI = 1:nRegions
                regionName = char(regionNames(currRegionI));
                currOutputDir = [outputDir strrep(regionName,' ','') '/'];
                mkdir(currOutputDir);
                currFN = [outputDir num2str(currRegionI) '.' strrep(regionName,' ','')  '.DailyData.csv'];
                currTable = readtable(currFN);
                allL3DateList = datenum(currTable{:,1},currTable{:,2},currTable{:,3});

                % for each background start year, calculate trends and
                % departure of last year from these trends etc, make annual
                % box plots
                for currBackGroundYearI = 1:nBackStartGroundYears
                    backGroundStartYear = backgroundStartYearList(currBackGroundYearI);
                    printRowI=(3*(currBackGroundYearI-1)+1):(3*(currBackGroundYearI-1)+3);
                    currTS = currTable{:,COStartCol+statOffSet};
                    currTSWeights = currTable{:,COStartCol+3};
                    if (logTransform)
                        nonPosI = find(currTS<=0);
                        currTS(nonPosI) = nanmin(currTS(find(currTS>0)));
                        currTS = log(currTS);
                    end
                        
                    currLetter = 'a) ';
                    currVarName = 'CO';
                    currSatName = 'AIRS';
                    currUnits = AIRSCOUnits;
                    precString = '%.1f';
                    [varNames,statsTableCO(currBackGroundYearI,:),varNamesPrint,statsTableCOPrint(printRowI,:),...
                        seasonalMeansCO,seasonalCountCO,seasonalMidPointDates] = DoDetrendedStatsAndPlots(currOutputDir,regionName,currLetter,currVarName,currSatName,currUnits,precString,statName,allL3DateList,currTS,currTSWeights,startYear,endYear,backGroundStartYear,backGroundEndYear,trendEndYear,...
                                                    lockDownStartMonth,lockDownStartDay,lockDownEndMonth,lockDownEndDay,regrModel,logTransform);

                    currTS = currTable{:,SO2StartCol+statOffSet};
                    currTSWeights = currTable{:,SO2StartCol+3};              
                    if (logTransform)
                        nonPosI = find(currTS<=0);
                        currTS(nonPosI) = nanmin(currTS(find(currTS>0)));
                        currTS = log(currTS);
                    end
                    currLetter = 'b) ';
                    currVarName = 'SO_2';
                    currSatName = 'OMI';
                    precString = '%.3f';
                    currUnits = OMISO2units;
                    [varNames,statsTableSO2(currBackGroundYearI,:),varNamesPrint,statsTableSO2Print(printRowI,:),...
                        seasonalMeansSO2,seasonalCountSO2,seasonalMidPointDates] = DoDetrendedStatsAndPlots(currOutputDir,regionName,currLetter,currVarName,currSatName,currUnits,precString,statName,allL3DateList,currTS,currTSWeights,startYear,endYear,backGroundStartYear,backGroundEndYear,trendEndYear,...
                                                    lockDownStartMonth,lockDownStartDay,lockDownEndMonth,lockDownEndDay,regrModel,logTransform);

                    currTS = currTable{:,NO2StartCol+statOffSet};
                    currTS = currTS/(10^15);
                    currTSWeights = currTable{:,NO2StartCol+3};              
                    if (logTransform)
                        nonPosI = find(currTS<=0);
                        currTS(nonPosI) = nanmin(currTS(find(currTS>0)));
                        currTS = log(currTS);
                    end
                    currLetter = 'c) ';
                    currVarName = 'NO_2';
                    currSatName = 'OMI';
                    precString = '%.1f';
                    currUnits = ['x10^{15} ' OMINO2units];
                    [varNames,statsTableNO2(currBackGroundYearI,:),varNamesPrint,statsTableNO2Print(printRowI,:),...
                        seasonalMeansNO2,seasonalCountNO2,seasonalMidPointDates] = DoDetrendedStatsAndPlots(currOutputDir,regionName,currLetter,currVarName,currSatName,currUnits,precString,statName,allL3DateList,currTS,currTSWeights,startYear,endYear,backGroundStartYear,backGroundEndYear,trendEndYear,...
                                                    lockDownStartMonth,lockDownStartDay,lockDownEndMonth,lockDownEndDay,regrModel,logTransform);

                    currTS_T = currTable{:,AODTStartCol+statOffSet};
                    currTSWeights_T = currTable{:,AODTStartCol+3}; 

                    currTS_A = currTable{:,AODAStartCol+statOffSet};
                    currTSWeights_A = currTable{:,AODAStartCol+3}; 

                    % calculate weighted average time of both instruments
                    currTS = NaN*ones(size(currTS_T,1),1);
                    currTSWeights = NaN*ones(size(currTS_T,1),1);

                    currLetter = 'd) ';
                    currVarName = 'AOD';
                    currSatName = 'MODIS';
                    currUnits = '';
                    precString = '%.2f';

                    for currDateI = 1:size(currTS_T,1)
                        currTS(currDateI) = nansum([currTS_T(currDateI)*currTSWeights_T(currDateI) currTS_A(currDateI)*currTSWeights_A(currDateI)])/nansum([currTSWeights_T(currDateI),currTSWeights_A(currDateI)]);
                        currTSWeights(currDateI) = nansum([currTSWeights_T(currDateI),currTSWeights_A(currDateI)]);
                    end    
                    
                    if (logTransform)
                        nonPosI = find(currTS<=0);
                        currTS(nonPosI) = nanmin(currTS(find(currTS>0)));
                        currTS = log(currTS);
                    end
                    [varNames,statsTableAOD(currBackGroundYearI,:),varNamesPrint,statsTableAODPrint(printRowI,:),...
                        seasonalMeansAOD,seasonalCountAOD,seasonalMidPointDates] = DoDetrendedStatsAndPlots(currOutputDir,regionName,currLetter,currVarName,currSatName,currUnits,precString,statName,allL3DateList,currTS,currTSWeights,startYear,endYear,backGroundStartYear,backGroundEndYear,trendEndYear,...
                                                    lockDownStartMonth,lockDownStartDay,lockDownEndMonth,lockDownEndDay,regrModel,logTransform);
                                                
                    % write out seasonal means
                    %
                    seasonalVarNames = {'YYYY','AIRS_CO','AIRS_CO_N','OMI_SO2','OMI_SO2_N','OMI_NO2','OMI_NO2_N','MODIS_AOD','MODIS_AOD_N'};
                    yearStr = cellstr(datestr(seasonalMidPointDates,'yyyy'));
                    outputTable = [yearStr num2cell(seasonalMeansCO') num2cell(seasonalCountCO') num2cell(seasonalMeansSO2') num2cell(seasonalCountSO2') num2cell(seasonalMeansNO2') num2cell(seasonalCountNO2') num2cell(seasonalMeansAOD') num2cell(seasonalCountAOD')];
                    outputTable = [seasonalVarNames; outputTable];
                             
                    outputTable = table(outputTable);
                    outFN = [outputDir 'SeasonalMeans.' statName '.' strrep(regionName,' ','') '.trendEndYear.' num2str(trendEndYear) '.' lockDownStr '.csv'];
                    writetable(outputTable,outFN,'WriteVariableNames',0); 
                                              
                                                
                end
                outputTable = [varNamesPrint;(statsTableCOPrint);(statsTableSO2Print);(statsTableNO2Print);(statsTableAODPrint)];
                rowNames = [cellstr('Variable') repmat(cellstr('CO'),3*size(statsTableNO2,1),1)' repmat(cellstr('SO2'),3*size(statsTableNO2,1),1)' ...
                                                repmat(cellstr('NO2'),3*size(statsTableNO2,1),1)' repmat(cellstr('AOD'),3*size(statsTableNO2,1),1)'];
                outputTable = [rowNames' outputTable];                            
                outputTable = table(outputTable);
                outFN = [outputDir 'Stats.Daily.' statName '.' strrep(regionName,' ','') '.trendEndYear.' num2str(trendEndYear) '.' lockDownStr '.' regrModel '.csv'];
                writetable(outputTable,outFN,'WriteVariableNames',0);
                
                % make plots of how stats depend on start of
                % background/detrend period
               
                close all;

                xMargin = 1;
                yMargin = 1;
                xGap=1.5;
                yGap=1;
                axWidth=5.5;
                axHeight=5.5;
                nCols = 2;
                nRows = 2;
                legGap = .01;                

                figH = figure('visible','off','PaperUnits','inches','Units','inches');
                %set(figH,'Resize','off');
                currPos = get(figH,'Position');
                figWidth = xMargin*2 + axWidth*(nCols) + xGap*(nCols-1) + legGap;
                figHeight = yMargin*2 + axHeight*(nRows) + yGap*(nRows-1) ; 

                set(figH,'Position',[currPos(1) currPos(2)  figWidth figHeight]);
                set(figH,'Color',[1 1 1]);% box off doesn't work,'box','off');
                
                dateRangeStr = [datestr(datenum(startYear,lockDownStartMonth,lockDownStartDay),'mmm dd') ...
                    ' to ' datestr(datenum(startYear,lockDownEndMonth,lockDownEndDay),'mmm dd')];
                
                currStatsTable = statsTableCO;
                currLetter = 'a) ';
                currVarName = 'CO';
                currSatName = 'AIRS';
                precString = '%.1f';
                currUnits = AIRSCOUnits;
                panelIndexCol=1;
                panelIndexRow=2;  
                MakeTrendAndDiffPlot(currStatsTable,regionName,currLetter,currVarName,currSatName,currUnits,precString,statName,...
                                     xMargin,yMargin,xGap,yGap,axWidth,axHeight,panelIndexCol,panelIndexRow,figH,dateRangeStr,logTransform,percDiff);

                currStatsTable = statsTableSO2;
                currLetter = 'b) ';
                currVarName = 'SO_2';
                currSatName = 'OMI';
                precString = '%.3f';
                currUnits = OMISO2units;
                panelIndexCol=2;
                panelIndexRow=2;  
                MakeTrendAndDiffPlot(currStatsTable,regionName,currLetter,currVarName,currSatName,currUnits,precString,statName,...
                                     xMargin,yMargin,xGap,yGap,axWidth,axHeight,panelIndexCol,panelIndexRow,figH,dateRangeStr,logTransform,percDiff);

                currStatsTable = statsTableNO2;
                currLetter = 'c) ';
                currVarName = 'NO_2';
                currSatName = 'OMI';
                precString = '%.1e';
                currUnits = ['x10^{15} ' OMINO2units];
                panelIndexCol=1;
                panelIndexRow=1;  
                MakeTrendAndDiffPlot(currStatsTable,regionName,currLetter,currVarName,currSatName,currUnits,precString,statName,...
                                     xMargin,yMargin,xGap,yGap,axWidth,axHeight,panelIndexCol,panelIndexRow,figH,dateRangeStr,logTransform,percDiff);

                currStatsTable = statsTableAOD;
                currLetter = 'd) ';
                currVarName = 'AOD';
                currSatName = 'MODIS';
                currUnits = '';
                precString = '%.3f';
                panelIndexCol=2;
                panelIndexRow=1;  
                MakeTrendAndDiffPlot(currStatsTable,regionName,currLetter,currVarName,currSatName,currUnits,precString,statName,...
                                     xMargin,yMargin,xGap,yGap,axWidth,axHeight,panelIndexCol,panelIndexRow,figH,dateRangeStr,logTransform,percDiff);

                imgFN = [outputDir  num2str(currRegionI) '.' strrep(regionName,' ','')  '.SlopeAndDiffStartYearPlot.'   statName  '.trendEndYear' num2str(trendEndYear) '.' lockDownStr '.' regrModel '.percDiff.' num2str(percDiff) '.png'];
                saveas(gcf,imgFN);                
                
                %
                % update table of stats for rows with maximum trend R2 -
                % didn't end up using this much in final version of paper
                %
                currR2 = statsTableCO(:,20);
                [maxR2,maxR2I] = max(currR2);
                maxStatsTable(4*(currRegionI-1)+1,:) = statsTableCO(maxR2I,:);
                currRowI = [3*(maxR2I-1)+1:3*(maxR2I-1)+3];
                destRowI = [12*(currRegionI-1)+1:12*(currRegionI-1)+3];
                maxStatsTablePrint(destRowI,:) = statsTableCOPrint(currRowI,:);

                currR2 = statsTableSO2(:,20);
                [maxR2,maxR2I] = max(currR2);
                maxStatsTable(4*(currRegionI-1)+2,:) = statsTableSO2(maxR2I,:);
                currRowI = [3*(maxR2I-1)+1:3*(maxR2I-1)+3];
                destRowI = [12*(currRegionI-1)+4:12*(currRegionI-1)+6];
                maxStatsTablePrint(destRowI,:) = statsTableSO2Print(currRowI,:);

                currR2 = statsTableNO2(:,20);
                [maxR2,maxR2I] = max(currR2);
                maxStatsTable(4*(currRegionI-1)+3,:) = statsTableNO2(maxR2I,:);
                currRowI = [3*(maxR2I-1)+1:3*(maxR2I-1)+3];
                destRowI = [12*(currRegionI-1)+7:12*(currRegionI-1)+9];
                maxStatsTablePrint(destRowI,:) = statsTableNO2Print(currRowI,:);
                                          
                currR2 = statsTableAOD(:,20);
                [maxR2,maxR2I] = max(currR2);
                maxStatsTable(4*(currRegionI-1)+4,:) = statsTableAOD(maxR2I,:);
                currRowI = [3*(maxR2I-1)+1:3*(maxR2I-1)+3];
                destRowI = [12*(currRegionI-1)+10:12*(currRegionI-1)+12];
                maxStatsTablePrint(destRowI,:) = statsTableAODPrint(currRowI,:);
                
            end
            
            % select columns for paper only
%            outputColumns = [1,3,5,7,9,10,12,14,19,22,27,28,30:34];
            outputTable = [varNamesPrint;(maxStatsTablePrint)];
            %outputTable = outputTable(:,outputColumns);
            rowNames = [ cellstr('CO (ppbv)')  cellstr('') cellstr('') cellstr('SO2 (DU) ') cellstr('') cellstr('') ...
                cellstr('NO2 (10^{15} molec cm^{-2})') cellstr('') cellstr('') cellstr('AOD')  cellstr('') cellstr('')];
            rowNames = [cellstr('Variable') repmat(rowNames,1,nRegions)];
            regionNameMat = reshape(repmat(regionNames,12,1),6*nRegions,2);
            regionNameMat = regionNameMat(:);
            rowNames = [[cellstr('Region') regionNameMat']',rowNames'];
            outputTable = [rowNames outputTable];                            
            outputTable = table(outputTable);
            outFN = [outputDir 'Stats.Max.Daily.' statName '.trendEndYear.' num2str(trendEndYear) '.' lockDownStr '.' regrModel '.csv'];
            writetable(outputTable,outFN,'WriteVariableNames',0);            
            
            
            % now assemble tiled image for max trend panels
            leftCropFac = .05; %.12;
            rightCropFac = .07;
            topCropFac =  .01; %.16;
            botCropFac = .01; %.21;              
            nTileRows=2;
            nTileCols = 2;
            
            for currRegionI = 1:nRegions
                regionName = char(regionNames(currRegionI));
                currOutputDir = [outputDir strrep(regionName,' ','') '/'];
                tiledImage = [];
                
                varName = 'CO';
                currStats = maxStatsTable(4*(currRegionI-1)+1,:);
                currMaxBackGroundYear = currStats(1);
                currFN = [currOutputDir strrep(regionName,' ','') '.' varName '.' statName '.BackGroundStartYear.' num2str(currMaxBackGroundYear) '.trendEndYear.' num2str(trendEndYear) '.' lockDownStr '.' regrModel '.stats.png'];
                currRowStart = 1;
                currColStart = 1;
                tiledImage = AddImageToPanel(currFN,tiledImage,currRowStart,currColStart,nTileRows,nTileCols,leftCropFac,rightCropFac,topCropFac,botCropFac);

                varName = 'SO_2';
                currStats = maxStatsTable(4*(currRegionI-1)+2,:);
                currMaxBackGroundYear = currStats(1);
                currFN = [currOutputDir strrep(regionName,' ','') '.' varName '.' statName '.BackGroundStartYear.' num2str(currMaxBackGroundYear) '.trendEndYear.' num2str(trendEndYear) '.' lockDownStr '.' regrModel  '.stats.png'];
                currRowStart = 1;
                currColStart = 2;
                tiledImage = AddImageToPanel(currFN,tiledImage,currRowStart,currColStart,nTileRows,nTileCols,leftCropFac,rightCropFac,topCropFac,botCropFac);

                varName = 'NO_2';
                currStats = maxStatsTable(4*(currRegionI-1)+3,:);
                currMaxBackGroundYear = currStats(1);
                currFN = [currOutputDir strrep(regionName,' ','') '.' varName '.' statName '.BackGroundStartYear.' num2str(currMaxBackGroundYear) '.trendEndYear.' num2str(trendEndYear) '.' lockDownStr '.' regrModel '.stats.png'];
                currRowStart = 2;
                currColStart = 1;
                tiledImage = AddImageToPanel(currFN,tiledImage,currRowStart,currColStart,nTileRows,nTileCols,leftCropFac,rightCropFac,topCropFac,botCropFac);

                varName = 'AOD';
                currStats = maxStatsTable(4*(currRegionI-1)+4,:);
                currMaxBackGroundYear = currStats(1);
                currFN = [currOutputDir strrep(regionName,' ','') '.' varName '.' statName '.BackGroundStartYear.' num2str(currMaxBackGroundYear) '.trendEndYear.' num2str(trendEndYear) '.' lockDownStr  '.' regrModel '.stats.png'];
                currRowStart = 2;
                currColStart = 2;
                tiledImage = AddImageToPanel(currFN,tiledImage,currRowStart,currColStart,nTileRows,nTileCols,leftCropFac,rightCropFac,topCropFac,botCropFac);

                image(tiledImage);
                currTileFileName = [outputDir num2str(currRegionI) '.' strrep(regionName,' ','') '.' statName  '.trendEndYear.' num2str(trendEndYear)  '.' lockDownStr '.' regrModel '.TiledMaxR2.png'];
                imwrite(tiledImage,currTileFileName);               
            end
        end
        
        % make the annual overlay plots (Fig 3,4)
        for currDateIncrementI = 1:nDateIncrements
            plotStartYear = plotStartYearList(currDateIncrementI);
            plotEndYear = plotEndYearList(currDateIncrementI);
            overlayStartMonth = overlayStartMonthList(currDateIncrementI);
            dateIncrement = dateIncrementList(currDateIncrementI);
            dateIncrementUnit = char(dateIncrementUnitList(currDateIncrementI));    
            titleDateFormat = char(titleDateFormatList(currDateIncrementI));

            for currRegionI = 1:nRegions
                regionName = char(regionNames(currRegionI));

                boxString = '';

                currFN = [outputDir num2str(currRegionI) '.' strrep(regionName,' ','')  '.DailyData.csv'];
                currTable = readtable(currFN);
                allL3DateList = datenum(currTable{:,1},currTable{:,2},currTable{:,3});
                allDateVec = datevec(allL3DateList);
                allDateVec = allDateVec(:,1:3);           

          
                close all;
                figH = figure('visible','off','PaperUnits','inches','Units','inches');
                %set(figH,'Resize','off');
                currPos = get(figH,'Position');
                figWidth = xMargin*2 + axWidth*(nCols) + xGap*(nCols-1) + legGap;
                figHeight = yMargin*2 + axHeight*(nRows) + yGap*(nRows-1) ; 


                set(figH,'Position',[currPos(1) currPos(2)  figWidth figHeight]);
                set(figH,'Color',[1 1 1]);% box off doesn't work,'box','off');

                xMargin = 1;
                yMargin = 1;
                xGap=1;
                yGap=1;
                axWidth=4;
                axHeight=4;
                nCols = 2;
                nRows = 2;
                legGap = 1;
                
                
                %
                % OMI NO2
                %
                currTS = currTable{:,NO2StartCol}/(10^15);
                currTSWeights = currTable{:,NO2StartCol+3};
                [allTimeStepAvg,allTimeStepDates] = CalculateTimeStepAverages(startDate,endDate,allL3DateList,currTS,currTSWeights,dateIncrement,dateIncrementUnit);

                [lastDateI,lastDateDummy] = max(find(~isnan(allTimeStepAvg)));
                lastDateStr = datestr(allTimeStepDates(lastDateI),titleDateFormat);
                yLabel = (['NO_2 (x10^{15}' OMINO2units ')' ]); %['x10^{15} ' OMINO2units]
                yLim = [NaN NaN]; %[.4*(10^16) 2.2*(10^16)]; %[0 1.1*nanmax(allTimeStepAvg)];
                %boxString = GetCoordBoxString(tsLatRange,tsLonRange);
                titleStr = [[cellstr('c) OMI NO_2')]; ...
                            cellstr([num2str(dateIncrement) '-' dateIncrementUnit ', ' regionName])];
                panelIndexCol=1;
                panelIndexRow=1;                
                PlotAnnualOverLay(allTimeStepDates,allTimeStepAvg,titleStr,yLabel,plotStartYear,plotEndYear,overlayStartMonth,dateIncrement,dateIncrementUnit,plotSmoothFactor,yLim,fontSize,...
                                xMargin,yMargin,xGap,yGap,axWidth,axHeight,panelIndexCol,panelIndexRow,figH,...
                                lockDownStartMonth,lockDownStartDay,lockDownEndMonth,lockDownEndDay);
               
                %
                % OMI SO2
                %
                currTS = currTable{:,SO2StartCol};
                currTSWeights = currTable{:,SO2StartCol+3};
                [allTimeStepAvg,allTimeStepDates] = CalculateTimeStepAverages(startDate,endDate,allL3DateList,currTS,currTSWeights,dateIncrement,dateIncrementUnit);

                [lastDateI,lastDateDummy] = max(find(~isnan(allTimeStepAvg)));
                lastDateStr = datestr(allTimeStepDates(lastDateI),titleDateFormat);
                yLabel = (['SO_2 (' OMISO2units ')' ]);
                yLim = [NaN NaN]; %[-.1 .8]; %[0 1.1*nanmax(allTimeStepAvg)];
                %boxString = GetCoordBoxString(tsLatRange,tsLonRange);
                titleStr = [[cellstr('b) OMI SO_2')]; ...
                            cellstr([num2str(dateIncrement) '-' dateIncrementUnit ', ' regionName])];
                panelIndexCol=2;
                panelIndexRow=2;                
                PlotAnnualOverLay(allTimeStepDates,allTimeStepAvg,titleStr,yLabel,plotStartYear,plotEndYear,overlayStartMonth,dateIncrement,dateIncrementUnit,plotSmoothFactor,yLim,fontSize,...
                                xMargin,yMargin,xGap,yGap,axWidth,axHeight,panelIndexCol,panelIndexRow,figH,...
                                lockDownStartMonth,lockDownStartDay,lockDownEndMonth,lockDownEndDay);
                                           
                %
                % AIRS CO
                %
                currTS = currTable{:,COStartCol};
                currTSWeights = currTable{:,COStartCol+3}; 
                [allTimeStepAvg,allTimeStepDates] = CalculateTimeStepAverages(startDate,endDate,allL3DateList,currTS,currTSWeights,dateIncrement,dateIncrementUnit);

                [lastDateI,lastDateDummy] = max(find(~isnan(allTimeStepAvg)));
                lastDateStr = datestr(allTimeStepDates(lastDateI),titleDateFormat);
                yLabel = ['CO (' AIRSCOUnits ') ' num2str(AIRSAnalysisPres) ' hPa'];
                yLim = [NaN NaN]; %[90 180]; %[40 1.1*nanmax(allTimeStepAvg)];
                %boxString = GetCoordBoxString(tsLatRange,tsLonRange);
                titleStr = [[cellstr('a) AIRS CO')]; ...
                            cellstr([num2str(dateIncrement) '-' dateIncrementUnit ', ' regionName])];
                panelIndexCol=1;
                panelIndexRow=2;                
                PlotAnnualOverLay(allTimeStepDates,allTimeStepAvg,titleStr,yLabel,plotStartYear,plotEndYear,overlayStartMonth,dateIncrement,dateIncrementUnit,plotSmoothFactor,yLim,fontSize,...
                                xMargin,yMargin,xGap,yGap,axWidth,axHeight,panelIndexCol,panelIndexRow,figH,...
                                lockDownStartMonth,lockDownStartDay,lockDownEndMonth,lockDownEndDay);
                
                %
                % MODIS AOD
                %
                currTS_T = currTable{:,AODTStartCol};
                currTSWeights_T = currTable{:,AODTStartCol+3}; 

                currTS_A = currTable{:,AODAStartCol};
                currTSWeights_A = currTable{:,AODAStartCol+3}; 

                % calculate weighted average time of both instruments
                currTS = NaN*ones(size(currTS_T,1),1);
                currTSWeights = NaN*ones(size(currTS_T,1),1);
                for currDateI = 1:size(currTS_T,1)
                    currTS(currDateI) = nansum([currTS_T(currDateI)*currTSWeights_T(currDateI) currTS_A(currDateI)*currTSWeights_A(currDateI)])/nansum([currTSWeights_T(currDateI),currTSWeights_A(currDateI)]);
                    currTSWeights(currDateI) = nansum([currTSWeights_T(currDateI),currTSWeights_A(currDateI)]);
                end

                [allTimeStepAvg,allTimeStepDates] = CalculateTimeStepAverages(startDate,endDate,allL3DateList,currTS,currTSWeights,dateIncrement,dateIncrementUnit);

                [lastDateI,lastDateDummy] = max(find(~isnan(allTimeStepAvg)));
                lastDateStr = datestr(allTimeStepDates(lastDateI),titleDateFormat);
                %boxString = GetCoordBoxString(tsLatRange,tsLonRange);
                yLabel = ('AOD');
                yLim = [NaN NaN]; %[0 1.2]; %[0 1.1*nanmax(allTimeStepAvg)];

                titleStr = [[cellstr('d) MODIS AOD')]; ...
                            cellstr([num2str(dateIncrement) '-' dateIncrementUnit ', ' regionName])];
                panelIndexCol=2;
                panelIndexRow=1;                
                PlotAnnualOverLay(allTimeStepDates,allTimeStepAvg,titleStr,yLabel,plotStartYear,plotEndYear,overlayStartMonth,dateIncrement,dateIncrementUnit,plotSmoothFactor,yLim,fontSize,...
                                xMargin,yMargin,xGap,yGap,axWidth,axHeight,panelIndexCol,panelIndexRow,figH,...
                                lockDownStartMonth,lockDownStartDay,lockDownEndMonth,lockDownEndDay);
             
                imgFN = [outputDir  num2str(currRegionI) '.' strrep(regionName,' ','')  '.AnnualOverlay.'  num2str(dateIncrement) '.' dateIncrementUnit  '.' datestr(plotStartDate,'yyyymmdd') '.' datestr(allTimeStepDates(lastDateI),'yyyymmdd')   '.' lockDownStr '.' regrModel '.png'];
                saveas(gcf,imgFN);   
            end
        end
    end
 

end

       
function [clippedMap] = trimMapToScene(dataGrid,sceneLon,sceneLat,gridLon,gridLat)

    clippedMap = NaN*dataGrid;
    goodI = find(~isnan(sceneLon(:)) & ~isnan(sceneLat(:)));
    if (~isempty(goodI))
        sceneShape = alphaShape(sceneLon(goodI),sceneLat(goodI),'RegionThreshold',0);
         %plot(K);
         
        isInSceneI = inpolygon(gridLon(:),gridLat(:),sceneShape.Points(:,1),sceneShape.Points(:,2));
        notInScenceI = ~isInSceneI; 
        clippedMap = dataGrid;
        clippedMap(notInScenceI) = NaN;                    
    end
end
                    
                    
function plotMap(projectionType,latLim,lonLim,Plg,Plt,plotField,titleStr,fieldColorScale,fontSize)
	mapAx = axesm('MapProjection',projectionType,... 'Xcolor','w','Ycolor','w',...
        'MapLatLim',latLim,...
        'MapLonLim',lonLim,...
        'Frame', 'off','FEdgeColor','k','FLineWidth',1,'FFaceColor',[255/255 255/255 255/255],...
        'Grid', 'on','MeridianLabel','off','ParallelLabel','off','MLabelParallel','south');    
    %set(gca,'box','off');
    sh = surfm(Plt,Plg,((plotField)),'AlphaData',~isnan((plotField))); 
    title(titleStr);
    geoshow('landareas.shp','FaceColor','none');
    caxis([fieldColorScale]);
    cbH=colorbar;
    cbH.Position = cbH.Position + 1e-10; %
    set(gca,'FontSize',fontSize);
    tightmap;
end



function [currMODISLat,currMODISLon,currMODISData] = getMODISL3Field(currMODISPath,landFieldName,oceanFieldName,landScaleFactor,oceanScaleFactor)
    % this merges land and ocean fields

    MODISOceanFracName = 'Ocean_Fraction_'; % Day,Night

    currMODISLat = double(hdfread(currMODISPath,'YDim'));
    currMODISLon = double(hdfread(currMODISPath,'XDim'));


    % should be reading these scaling and missing values from file....later
    currMODISDataLand = double(hdfread(currMODISPath,landFieldName));
    currMODISDataLand(find(currMODISDataLand == -9999)) = NaN;
    currMODISDataLand = double(currMODISDataLand);
    currMODISDataLand = currMODISDataLand*landScaleFactor;

    currMODISData = currMODISDataLand;
    
    if (~strcmp(landFieldName,oceanFieldName))
        currMODISDataOcean = double(hdfread(currMODISPath,oceanFieldName));
        currMODISDataOcean(find(currMODISDataOcean == -9999)) = NaN;
        currMODISDataOcean = double(currMODISDataOcean);
        currMODISDataOcean = currMODISDataOcean*oceanScaleFactor;

        currMODISOceanFracDay = double(hdfread(currMODISPath,[MODISOceanFracName 'Day']));            
        currMODISOceanFracNight = double(hdfread(currMODISPath,[MODISOceanFracName 'Night']));            
        currMODISOceanFrac(1,:,:) = 9.999999747378752E-5*currMODISOceanFracDay;
        currMODISOceanFrac(2,:,:) = 9.999999747378752E-5*currMODISOceanFracNight;
        currMODISOceanFrac = squeeze(nanmean(currMODISOceanFrac,1));
    
        % replace ocean retrieval with land retrieval over land cells
        oceanI = find(currMODISOceanFrac >0.5);
        currMODISData(oceanI) = currMODISDataOcean(oceanI);

        %badDataI = find(isnan(currMODISLat) | isnan(currMODISLon));
        %currMODISData(badDataI) = NaN;

        currMODISData = double(currMODISData);
    end
end


 

function [cmap]=buildcmap(colors)
% [cmap]=buildcmap(colors)
%
% This function can be used to build your own custom colormaps. Imagine if
% you want to display rainfall distribution map. You want a colormap which
% ideally brings rainfall in mind, which is not achiveved by colormaps such
% as winter, cool or jet and such. A gradient of white to blue will do the
% task, but you might also use a more complex gradient (such as
% white+blue+red or colors='wbr'). This function can be use to build any
% colormap using main colors rgbcmyk. In image processing, w (white) can be
% used as the first color so that in the output, the background (usually
% with 0 values) appears white. In the example of rainfall map, 'wb' will
% produce a rainfall density map where the background (if its DN values are
% 0) will appear as white.
%
% Inputs:
%  colors: string (char) of color codes, any sequence of rgbcmywk
%  representing different colors (such as 'b' for blue) is acceptable. If a
%  gradient of white to blue is needed, colors would be 'wb'; a rainbow of
%  white+blue+red+green would be 'wbrg'.
%
% Example:
%  [cmap]=buildcmap('wygbr');
% %try the output cmap:
% im=imread('cameraman.tif');
% imshow(im), colorbar
% colormap(cmap) %will use the output colormap
%
% First version: 14 Feb. 2013
% sohrabinia.m@gmail.com
%--------------------------------------------------------------------------

if nargin<1
    colors='wrgbcmyk';
end

if ~ischar(colors)
    error(['Error! colors must be a variable of type char with '...
        'color-names, such as ''r'', ''g'', etc., '...
        'type ''help buildcmap'' for more info']);
end

ncolors=length(colors)-1;


bins=round(255/ncolors);
% diff1=255-bins*ncolors;

vec=zeros(300,3);

switch colors(1)
    case 'w'
        vec(1,:)=1;
    case 'r'
        vec(1,:)=[1 0 0];
    case 'g'
        vec(1,:)=[0 1 0];
    case 'b'
        vec(1,:)=[0 0 1];
    case 'c'
        vec(1,:)=[0 1 1];
    case 'm'
        vec(1,:)=[1 0 1];
    case 'y'
        vec(1,:)=[1 1 0];
    case 'k'
        vec(1,:)=[0 0 0];
end


for i=1:ncolors
 beG=(i-1)*bins+1;
 enD=i*bins+1; %beG,enD
 switch colors(i+1)
     case 'w'
         vec(beG:enD,1)=linspace(vec(beG,1),1,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),1,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),1,bins+1)';%colors(i+1),beG,enD,
     case 'r'
         vec(beG:enD,1)=linspace(vec(beG,1),1,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),0,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),0,bins+1)';%colors(i+1),beG,enD
     case 'g'
         vec(beG:enD,1)=linspace(vec(beG,1),0,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),1,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),0,bins+1)';%colors(i+1),beG,enD
     case 'b'         
         vec(beG:enD,1)=linspace(vec(beG,1),0,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),0,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),1,bins+1)';%colors(i+1),beG,enD
     case 'c'
         vec(beG:enD,1)=linspace(vec(beG,1),0,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),1,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),1,bins+1)';%colors(i+1),beG,enD
     case 'm'
         vec(beG:enD,1)=linspace(vec(beG,1),1,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),0,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),1,bins+1)';
     case 'y'
         vec(beG:enD,1)=linspace(vec(beG,1),1,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),1,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),0,bins+1)';
     case 'k'
         vec(beG:enD,1)=linspace(vec(beG,1),0,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),0,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),0,bins+1)';
 end
end
cmap=vec(1:bins*ncolors,:);
end %end of buildcmap



function [ha, pos] = tight_subplot(Nh, Nw, gap, marg_h, marg_w)

% tight_subplot creates "subplot" axes with adjustable gaps and margins
%
% [ha, pos] = tight_subplot(Nh, Nw, gap, marg_h, marg_w)
%
%   in:  Nh      number of axes in hight (vertical direction)
%        Nw      number of axes in width (horizontaldirection)
%        gap     gaps between the axes in normalized units (0...1)
%                   or [gap_h gap_w] for different gaps in height and width 
%        marg_h  margins in height in normalized units (0...1)
%                   or [lower upper] for different lower and upper margins 
%        marg_w  margins in width in normalized units (0...1)
%                   or [left right] for different left and right margins 
%
%  out:  ha     array of handles of the axes objects
%                   starting from upper left corner, going row-wise as in
%                   subplot
%        pos    positions of the axes objects
%
%  Example: ha = tight_subplot(3,2,[.01 .03],[.1 .01],[.01 .01])
%           for ii = 1:6; axes(ha(ii)); plot(randn(10,ii)); end
%           set(ha(1:4),'XTickLabel',''); set(ha,'YTickLabel','')

% Pekka Kumpulainen 21.5.2012   @tut.fi
% Tampere University of Technology / Automation Science and Engineering


if nargin<3; gap = .02; end
if nargin<4 || isempty(marg_h); marg_h = .05; end
if nargin<5; marg_w = .05; end

if numel(gap)==1; 
    gap = [gap gap];
end
if numel(marg_w)==1; 
    marg_w = [marg_w marg_w];
end
if numel(marg_h)==1; 
    marg_h = [marg_h marg_h];
end

axh = (1-sum(marg_h)-(Nh-1)*gap(1))/Nh; 
axw = (1-sum(marg_w)-(Nw-1)*gap(2))/Nw;

py = 1-marg_h(2)-axh; 

% ha = zeros(Nh*Nw,1);
ii = 0;
for ih = 1:Nh
    px = marg_w(1);
    
    for ix = 1:Nw
        ii = ii+1;
        ha(ii) = axes('Units','normalized', ...
            'Position',[px py axw axh], ...
            'XTickLabel','', ...
            'YTickLabel','');
        px = px+axw+gap(2);
    end
    py = py-axh-gap(1);
end
if nargout > 1
    pos = get(ha,'Position');
end
ha = ha(:);

end



function [allTimeStepAvg,allTimeStepDates] = CalculateTimeStepAverages(startDate,endDate,allL3DateList,dailyTimeSeries,dailyWeight,dateIncrement,dateIncrementUnit)
    allTimeStepAvg = [];
    allTimeStepDates = [];
    currDate = startDate;
    currStepI = 1;
    while (currDate <=endDate)
        allTimeStepDates(currStepI) = currDate;
        currEndDate = addtodate(currDate,dateIncrement,dateIncrementUnit);
        currDateI = find(allL3DateList >= currDate & allL3DateList < currEndDate);

        currAvg = dailyTimeSeries(currDateI);
        currCount = dailyWeight(currDateI);
        goodI = find(~isnan(currAvg) & ~isnan(currCount));
        if (~isempty(goodI))
            currAvg = wmean(currAvg(goodI),currCount(goodI));
            allTimeStepAvg(currStepI) = currAvg;
        else
            allTimeStepAvg(currStepI) = NaN;

        end

        currDate = addtodate(currDate,dateIncrement,dateIncrementUnit);
        currStepI = currStepI+1;
    end
end


function PlotAnnualOverLay(allTimeStepDates,allTimeStepAvg,titleStr,yLabel,startYear,endYear,overlayStartMonth,dateIncrement,dateIncrementUnit,plotSmoothFactor,yLim,fontSize,...
                            xMargin,yMargin,xGap,yGap,axWidth,axHeight,panelIndexCol,panelIndexRow,figH,...
                            lockDownStartMonth,lockDownStartDay,lockDownEndMonth,lockDownEndDay)

	set(0, 'currentfigure', figH);
    currSubAx = axes();
    hold on;
    if (overlayStartMonth>1)
        plotYearList = [startYear:endYear-1];
    else
        plotYearList = [startYear:endYear];
    end
  
    
    nPlotYears = size(plotYearList,2);
	%annualLineColors = jet(nPlotYears-1); 
    annualLineColors = buildcmap('bwr');
    nColors = size(annualLineColors,1);
    sampleInterval = ceil(nColors/nPlotYears);;
    annualLineColors = annualLineColors(1:sampleInterval:end,:);
            % make last one black
            
    annualLineColors(ceil(nPlotYears/2),:) = [.8 .8 .8];        
    annualLineColors(nPlotYears-1,:) = [255 0 255]/256;    
    annualLineColors(nPlotYears,:) = [0 0 0];    
    
    
    for currYearI = 1:nPlotYears
        currYear = plotYearList(currYearI);
        fakeDateStart = datenum(startYear,overlayStartMonth,1);
        fakeDateEnd = addtodate(fakeDateStart,12,'month');
        fakeDateEnd = addtodate(fakeDateEnd,-1,'day');

        clear fakeDateNums;
        currFakeDateNum = fakeDateStart;
        currFakeStepI = 1;
        while(currFakeDateNum < fakeDateEnd)
            fakeDateNums(currFakeStepI) = currFakeDateNum;

            currFakeStepI = currFakeStepI + 1;
            currFakeDateNum = addtodate(currFakeDateNum,dateIncrement,dateIncrementUnit);
        end

        fakeDateVec = datevec(fakeDateNums);
        %currI = find(~(fakeDateVec(:,2) == 2 & fakeDateVec(:,3) == 29));
        %fakeDateNums = fakeDateNums(currI);

        currStartDate = datenum(currYear,overlayStartMonth,1);
        currEndDate = addtodate(currStartDate,1,'year');

        currI = find(allTimeStepDates >= currStartDate & allTimeStepDates < currEndDate );
        currPlotTS = allTimeStepAvg(currI);
        %currNSteps = size(currPlotTS,1);
        %fakeDateNums = fakeDateNums(1:currNSteps);    
        % smoothed plot
        smoothDailyPlot = smooth(currPlotTS,plotSmoothFactor,'moving');

        goodDayI = find(~isnan(smoothDailyPlot));

        plotDates = fakeDateNums + (fakeDateNums(2)-fakeDateNums(1))/2;
        endI = min(size(plotDates,2),size(smoothDailyPlot,1));
        if (~isempty(goodDayI))
            if (currYear < endYear)
                tsAx = plot(plotDates(1:endI),smoothDailyPlot(1:endI),'Color',annualLineColors(currYearI,:),'LineWidth',2);
            else
                tsAx = plot(plotDates(1:endI),smoothDailyPlot(1:endI),'Color',annualLineColors(currYearI,:),'LineWidth',4);
            end
            if (overlayStartMonth ==1)
                legendYearList(currYearI) = cellstr(num2str(currYear));
            else
                legendYearList(currYearI) = cellstr([num2str(currYear) '/' datestr(currEndDate,'yy') ]);
            end
        end
    end


    if (isempty(find(isnan(yLim))))
        ylim(yLim);
    end
    currYLim = get(gca,'YLim');

    newYLim = get(gca,'YLim');
    % draw lockdown period vertical lines
    line([datenum(startYear+1,lockDownStartMonth,lockDownStartDay) datenum(startYear+1,lockDownStartMonth,lockDownStartDay)],...
         [newYLim(1) newYLim(2)],'LineStyle','-','Color',[.5 .5 .5]);
    line([datenum(startYear+1,lockDownEndMonth,lockDownEndDay) datenum(startYear+1,lockDownEndMonth,lockDownEndDay)],...
         [newYLim(1) newYLim(2)],'LineStyle','-','Color',[.5 .5 .5]);
    set(gca,'YLim',newYLim);
        datetick;
    
    plotStartDate = min(fakeDateNums);
    plotEndDate = max(fakeDateEnd);
    xlim([plotStartDate plotEndDate]);
    %xlabel('Year');
    ylabel(yLabel);

    if (panelIndexCol == 2)
        legH = legend(legendYearList,'Location','EastOutside');
        set(legH,'box','off');
    end
    set(gca,'FontSize',fontSize);
    title(titleStr,'FontSize',fontSize);            

     set(currSubAx,'Color',[1 1 1]);% box off doesn't work,'box','off')
 %   set(currSubAx,'XColor',get(figH,'color'));
 %   set(currSubAx,'YColor',get(figH,'color'));
    set(currSubAx,'Units','inches');                
    currXPos = (xMargin +  (xGap+axWidth)*(panelIndexCol - 1)) ;
    currYPos = (yMargin + (yGap+axHeight)*(panelIndexRow -1 )) ;
    newAxPos = [currXPos currYPos axWidth axHeight];
    set(currSubAx,'Position',newAxPos);    
    
  
end


function [regionNames,regionZones,shapeArray,YI,XI] = DefineGriddedShapesADM1(latVec,lonVec,shapefilePath,regionNames,regionZones)
%
% This takes lat/lon vectors and returns grids assigning a code to each
% element of regionNames in shapeArrays, along with the mesgrid style lon
% (YI) and lat (XI) arrays
%
    nGroups = size(regionNames,2);
    currRegions =  shaperead(shapefilePath,'UseGeoCoords',true);
    nRegions = size(currRegions,1);
   
   
    for currProvI = 1:nRegions
        allZoneNames(currProvI,1) = cellstr(currRegions(currProvI).NAME_1);
        % special case
        currLats = currRegions(currProvI).Lat';
        currLons = currRegions(currProvI).Lon';
        fireZoneShapes{currProvI,1} = [currLats currLons ];
 
        goodI = find(~isnan(currLats) & ~isnan(currLons));

        %currPoly = polyshape(currX,currY);
        %[x,y] = centroid(currPoly);
        [ geom, iner, cpmo ] = polygeom(currLons(goodI), currLats(goodI));
        fireZoneCentroids(currProvI,1) = geom(3);
        fireZoneCentroids(currProvI,2) = geom(2);
    end

    goodI = [1:nRegions]; %find(contains(allFireCentres,'Kalimantan Barat'));
    


  	[YI,XI] = meshgrid(latVec,lonVec);

    clear allIndexLists;
    for currGroupI = 1:nGroups
        currNZones = size(regionZones(currGroupI).List,2);
        newList = [];
        for currZoneListI = 1:currNZones
            currZoneName = regionZones(currGroupI).List(currZoneListI);
            currZoneI = find(strcmp(allZoneNames,currZoneName));
            currPoly = fireZoneShapes{currZoneI,1};

            % assign wx grid cells to zone
            [inI] = find(inpolygon(YI(:),XI(:),currPoly(:,1),currPoly(:,2)));
            if (~isempty(inI))
                newList = [newList inI'];
            end
        end
        allIndexLists(currGroupI).IndexList =newList;
    end
    
    shapeArray = 0*XI-1;
    for currGroupI = 1:nGroups
        inI =  allIndexLists(currGroupI).IndexList;
        shapeArray(inI) = currGroupI;
    end
    %figure('visible','on')
    %surface(shapeArray,'EdgeColor','none');

end  


function [varNames,statsTable,varNamesPrint,statsTablePrint,seasonalMeans,seasonalCounts,seasonalMidPointDates] = DoDetrendedStatsAndPlots(outputDir,regionName,currLetter,currVarName,currSatName,currUnits,precString,statName,allL3DateList,...
                                                                                                        currTS,currTSWeights,startYear,endYear,backGroundStartYear,backGroundEndYear,trendEndYear,...
                                                                                                        lockDownStartMonth,lockDownStartDay,lockDownEndMonth,lockDownEndDay,regrModel,logTransform)
   % 
   % This calculates trends and makes the box plots (Fig 5,7) and SI tables 1-8. There's commented out
   % code plot the trends or make detrended plots. 
   %
	lockDownStr = ['lockDown.' datestr(datenum(startYear,lockDownStartMonth,lockDownStartDay),'mm.dd') '.' ...
                               datestr(datenum(startYear,lockDownEndMonth,lockDownEndDay),'mm.dd')];
                           
	nSamples = 2000;
    yearList = [startYear:endYear];

    % accumulate indices for full period, background period, year previous to lockdown,
    % lockdown year
	fullPeriodI = [];
    backGroundPeriodI = [];
    prevYearI = [];
    lockDownYearI = [];
	seasonalMeans = [];
    seasonalMidPointDates = [];
    seasonalCounts = [];

    
    for currYear = startYear:endYear
        currBackGroundStartDate = datenum(currYear,lockDownStartMonth,lockDownStartDay);
        currBackGroundEndDate = datenum(currYear,lockDownEndMonth,lockDownEndDay);
        currI = find(allL3DateList>=currBackGroundStartDate & allL3DateList <=currBackGroundEndDate  & ~isnan(currTS));
        
        seasonalMeans(currYear-startYear+1) = wmean(currTS(currI),currTSWeights(currI));

        seasonalCounts(currYear-startYear+1)= nansum(currTSWeights(currI));
        seasonalMidPointDates(currYear-startYear+1)= nanmean(allL3DateList(currI));
        
        fullPeriodI = [fullPeriodI; currI];
        if (currYear >=backGroundStartYear & currYear <=backGroundEndYear)
            backGroundPeriodI = [backGroundPeriodI; currI];
            indexLists(currYear-startYear+1).backGroundI = currI;
        end
        if (currYear == endYear-1)
            prevYearI = [prevYearI; currI];
            indexLists(currYear-startYear+1).prevYearI = currI;
        end      
        if (currYear ==endYear)
            lockDownYearI = [lockDownYearI; currI];
            indexLists(currYear-startYear+1).lockDownI = currI;
        end
    end
    
    nFullPeriod = size(fullPeriodI,1);
    nBackGroundPeriod = size(backGroundPeriodI,1);
    nPrevYear = size(prevYearI,1);
    nLockDown = size(lockDownYearI,1);
                               
    nTrendYears = trendEndYear-backGroundStartYear+1;
    trendYearI = find(yearList>=backGroundStartYear & yearList<=trendEndYear);
    
    %  calculate means for the different periods, % diff between 2020,
    % trends and 2020 prediction from those trends
    parfor currSampleI = 1:nSamples
        
        % means over lockdown, previous year, background
        currLockDownSampleI = randsample((lockDownYearI),nLockDown,true);
        lockDownSamples(currSampleI) = wmean(currTS(currLockDownSampleI),currTSWeights(currLockDownSampleI));
        
        currPrevYearSampleI = randsample((prevYearI),nPrevYear,true);
        prevYearSamples(currSampleI) = wmean(currTS(currPrevYearSampleI),currTSWeights(currPrevYearSampleI));

        currBackGroundSampleI = randsample((backGroundPeriodI),nBackGroundPeriod,true);
        backGroundSamples(currSampleI) = wmean(currTS(currBackGroundSampleI),currTSWeights(currBackGroundSampleI));
        
        % trends over background period predictions for 2020
        regX = allL3DateList(currBackGroundSampleI);
        regY = currTS(currBackGroundSampleI);
        regWeight = currTSWeights(currBackGroundSampleI);
        [fitresult,gofResult] = fit(regX,regY,regrModel,'Weight',regWeight);
        xPred = allL3DateList(currLockDownSampleI);
        fitPred =  feval(fitresult,xPred);
        lockDownPredSamples(currSampleI) = nanmean(fitPred);
        
        % trends on averages for reporting
        %currTrendYearSampleI = sort(randsample(trendYearI,nTrendYears,true));
        %regX = seasonalMidPointDates(currTrendYearSampleI)';
        %regY = seasonalMeans(currTrendYearSampleI)';
        %[fitresult,gofResult] = fit(regX,regY,'poly1');

        annualTrendSamples(currSampleI) = 365*fitresult.p1;
        annualTrendR2Samples(currSampleI) = gofResult.rsquare;
        
    end
	% extract means and 95% CI on all stats
    
    lockDownMean = nanmean(lockDownSamples);
    lockDownCI = prctile(lockDownSamples,[2.5,97.5]);
    
    prevYearMean = nanmean(prevYearSamples);
    prevYearCI = prctile(prevYearSamples,[2.5,97.5]);
    prevYearPercDiffs =100*(lockDownSamples-prevYearSamples)./abs(prevYearSamples);
    prevYearPercDiffMean = nanmean(prevYearPercDiffs);
    prevYearPercDiffCI = prctile(prevYearPercDiffs,[2.5,97.5]);

    backGroundMean = nanmean(backGroundSamples);
    backGroundCI = prctile(backGroundSamples,[2.5,97.5]);
    backGroundPercDiffs =100*(lockDownSamples-backGroundSamples)./abs(backGroundSamples);
    backGroundPercDiffMean = nanmean(backGroundPercDiffs);
    backGroundPercDiffCI = prctile(backGroundPercDiffs,[2.5,97.5]);
    
    trendMean = nanmean(annualTrendSamples);
    trendCI = prctile(annualTrendSamples,[2.5,97.5]);
    
    trendR2Mean = nanmean(annualTrendR2Samples);
    trendR2CI = prctile(annualTrendR2Samples,[2.5,97.5]);
    
    trendPredMean = nanmean(lockDownPredSamples);
    trendPredCI = prctile(lockDownPredSamples,[2.5,97.5]);
    trendPredPercDiffs =100*(lockDownSamples-lockDownPredSamples)./abs(lockDownPredSamples);
    trendPredPercDiffMean = nanmean(trendPredPercDiffs);
    trendPredPercDiffCI = prctile(trendPredPercDiffs,[2.5,97.5]);
    
    trendPredAbsDiff = lockDownSamples-lockDownPredSamples;
    trendPredAbsDiffMean = nanmean(trendPredAbsDiff);
    trendPredAbsDiffCI = prctile(trendPredAbsDiff,[2.5,97.5]);
        
                               
	% make box plot time series
    titleVarName = [currLetter currSatName ' ' currVarName];
    varName = currVarName;

    if (logTransform)
        yLabel = [currSatName ' ln(' currVarName ')' ];
    else
        yLabel = [currSatName ' ' currVarName ];
    end
    if (~strcmp(currUnits,''))
       yLabel = [yLabel ' (' currUnits ')']; 
    end
    
    dateRangeStr = [datestr(datenum(startYear,lockDownStartMonth,lockDownStartDay),'mmm dd ') ...
                    ' to ' datestr(datenum(startYear,lockDownEndMonth,lockDownEndDay),'mmm dd ')];
    maxNDays = datenum(startYear,lockDownEndMonth,lockDownEndDay) - datenum(startYear,lockDownStartMonth,lockDownStartDay) + 2;
    
    
    close all;
    figH = figure('visible','off');
	set(0, 'currentfigure', figH); 
    figPos = get(figH,'Position');
	set(figH,'Position',[figPos(1) figPos(2) figPos(3) figPos(4) ]);

    hold on;
    title([cellstr([titleVarName ' ' regionName ', ' dateRangeStr])]);

    % calculate a single nonbootstrap trend on daily data for plotting 
    regX = allL3DateList(backGroundPeriodI);
    regY = currTS(backGroundPeriodI);
    regWeight = currTSWeights(backGroundPeriodI);

    [fitresult,gofResult] = fit(regX,regY,regrModel,'Weight',regWeight);
    slopeCI = confint(fitresult);
    annualTrend = 365*fitresult.p1;
    annualTrendLCL = 365*slopeCI(1,1);
    annualTrendUCL = 365*slopeCI(2,1);

    xPred = allL3DateList(backGroundPeriodI);
    fitPred =  feval(fitresult,xPred);
    %scatter(xPred,fitPred,16,'filled','MarkerFaceColor','r','MarkerEdgeColor','r');
    
    xPred = allL3DateList(lockDownYearI);
    fitPred =  feval(fitresult,xPred);
	%scatter(xPred,fitPred,16,'filled','MarkerFaceColor','b','MarkerEdgeColor','b');
    scatter(seasonalMidPointDates,seasonalMeans,16,'filled','MarkerFaceColor','k','MarkerEdgeColor','k');
    
    % 
    for currYear = startYear:endYear
        currBackGroundStartDate = datenum(currYear,lockDownStartMonth,lockDownStartDay);
        currBackGroundEndDate = datenum(currYear,lockDownEndMonth,lockDownEndDay);
        currI = find(allL3DateList>=currBackGroundStartDate & allL3DateList <=currBackGroundEndDate  & ~isnan(currTS));

        % raw box plots
        currPlotData = currTS(currI);
        currPercIles = prctile(currPlotData,[2.5,25,50,75,97.5]);
        line([currBackGroundStartDate,currBackGroundEndDate],[currPercIles(1), currPercIles(1)],'Color','k','LineWidth',1);
        line([currBackGroundStartDate,currBackGroundEndDate],[currPercIles(2), currPercIles(2)],'Color','k','LineWidth',1);
        line([currBackGroundStartDate,currBackGroundEndDate],[currPercIles(3), currPercIles(3)],'Color','k','LineWidth',1);
        line([currBackGroundStartDate,currBackGroundEndDate],[currPercIles(4), currPercIles(4)],'Color','k','LineWidth',1);
        line([currBackGroundStartDate,currBackGroundEndDate],[currPercIles(5), currPercIles(5)],'Color','k','LineWidth',1);
        
        line([currBackGroundStartDate currBackGroundStartDate],[currPercIles(2), currPercIles(4)],'Color','k','LineWidth',1);
        line([currBackGroundEndDate currBackGroundEndDate],[currPercIles(2), currPercIles(4)],'Color','k','LineWidth',1);

        line([nanmean(allL3DateList(currI)) nanmean(allL3DateList(currI))],[currPercIles(1), currPercIles(2)],'Color','k','LineWidth',1);
        line([nanmean(allL3DateList(currI)) nanmean(allL3DateList(currI))],[currPercIles(4), currPercIles(5)],'Color','k','LineWidth',1);
        
        % detrended box plots
        %{
        if (currYear>=backGroundStartYear)
            xPred = allL3DateList(currI);
            fitPred =  feval(fitresult,xPred);
            currPlotData = currTS(currI)-fitPred;
            currPercIles = prctile(currPlotData,[2.5,25,50,75,97.5]);
            line([currBackGroundStartDate,currBackGroundEndDate],[currPercIles(1), currPercIles(1)],'Color',[255 0 255]/256,'LineWidth',1);
            line([currBackGroundStartDate,currBackGroundEndDate],[currPercIles(2), currPercIles(2)],'Color',[255 0 255]/256,'LineWidth',1);
            line([currBackGroundStartDate,currBackGroundEndDate],[currPercIles(3), currPercIles(3)],'Color',[255 0 255]/256,'LineWidth',1);
            line([currBackGroundStartDate,currBackGroundEndDate],[currPercIles(4), currPercIles(4)],'Color',[255 0 255]/256,'LineWidth',1);
            line([currBackGroundStartDate,currBackGroundEndDate],[currPercIles(5), currPercIles(5)],'Color',[255 0 255]/256,'LineWidth',1);

            line([currBackGroundStartDate currBackGroundStartDate],[currPercIles(2), currPercIles(4)],'Color',[255 0 255]/256,'LineWidth',1);
            line([currBackGroundEndDate currBackGroundEndDate],[currPercIles(2), currPercIles(4)],'Color',[255 0 255]/256,'LineWidth',1);

            line([nanmean(allL3DateList(currI)) nanmean(allL3DateList(currI))],[currPercIles(1), currPercIles(2)],'Color',[255 0 255]/256,'LineWidth',1);
            line([nanmean(allL3DateList(currI)) nanmean(allL3DateList(currI))],[currPercIles(4), currPercIles(5)],'Color',[255 0 255]/256,'LineWidth',1);
        end        
        %}
    end

    datetick('x','yy')

    xlabel('Year');
    ylabel(yLabel);
    xlim([datenum(startYear,1,1) datenum(endYear,12,31)]);%[min(allL3DateList) max(allL3DateList)]);
    set(gca,'YTickLabelMode','auto');
    topYLim=get(gca,'YLim');

    
    currXLim = get(gca,'XLim');
    currXPos = currXLim(1) + .60*(currXLim(2)-currXLim(1));
    currYLim = get(gca,'YLim');
    
    % add a bit to top of scale
    yWidth = [currYLim(2)-currYLim(1)];
    currYLim = [currYLim(1)-.01*yWidth currYLim(2)+.1*yWidth];
    set(gca,'YLim',currYLim);
    
    % add stats strings

    textStr = cellstr([num2str(endYear) ' difference from ' num2str(endYear-1) ': ' num2str(nanmean(prevYearPercDiffMean),'%.1f') '%'  ... 
                      ' (' num2str(prevYearPercDiffCI(1),'%.1f') ', '  num2str(prevYearPercDiffCI(2),'%.1f') ')']);
    currYPos = currYLim(1) + .99*(currYLim(2)-currYLim(1));
    %text(currXPos,currYPos,textStr,'Color','k','HorizontalAlignment','center');

    textStr = cellstr([num2str(endYear) ' difference from ' num2str(backGroundStartYear) ' to ' num2str(backGroundEndYear) ': ' num2str(nanmean(backGroundPercDiffMean),'%.1f') '%'  ... 
                      ' (' num2str(backGroundPercDiffCI(1),'%.1f') ', '  num2str(backGroundPercDiffCI(2),'%.1f') ')']);
    currYPos = currYLim(1) + .94*(currYLim(2)-currYLim(1));
    %text(currXPos,currYPos,textStr,'Color','k','HorizontalAlignment','center');
    
    textStr = cellstr(['trend: ' num2str(trendMean,precString) ' ' currUnits ' yr^{-1} (' num2str(trendCI(1),precString) ', ' num2str(trendCI(2),precString) ...
                '), r^2= ' num2str(trendR2Mean,'%.2f') ' (' num2str(trendR2CI(1),'%.2f') ', ' num2str(trendR2CI(2),'%.2f') ')']);
    currYPos = currYLim(1) + .89*(currYLim(2)-currYLim(1));
    %text(currXPos,currYPos,textStr,'Color','r','HorizontalAlignment','center');

    textStr = cellstr([num2str(endYear) ' difference from trend: ' num2str(nanmean(trendPredPercDiffMean),'%.1f') '%'  ... 
                      ' (' num2str(trendPredPercDiffCI(1),'%.1f') ', '  num2str(trendPredPercDiffCI(2),'%.1f') ')']);
    currYPos = currYLim(1) + .84*(currYLim(2)-currYLim(1));
    %text(currXPos,currYPos,textStr,'Color','b','HorizontalAlignment','center');
    
    currFN = [outputDir strrep(regionName,' ','') '.' varName '.' statName '.BackGroundStartYear.' num2str(backGroundStartYear) '.trendEndYear.' num2str(trendEndYear) '.' lockDownStr '.' regrModel '.stats.png'];
    saveas(gcf,currFN);
    
    varNames = {'Background start year' ...
                '2020 mean' 'LCL' 'UCL' ...
                '2019 mean' 'LCL' 'UCL' ...
                '2020 % difference from 2019' 'LCL' 'UCL' ...
                'Background mean' 'LCL' 'UCL' ...
                '2020 difference from background' 'LCL' 'UCL' ...
                'trend' 'LCL' 'UCL' ...
                'trend R2' 'LCL' 'UCL' ...
                '2020 predicted from trend' 'LCL' 'UCL' ...
                '2020 % difference from trend' 'LCL' 'UCL' ...
                '2020 difference from trend' 'LCL' 'UCL' ...
        };
    statsTable = [backGroundStartYear ...
                  lockDownMean lockDownCI ...
                  prevYearMean prevYearCI ...
                  prevYearPercDiffMean prevYearPercDiffCI ...
                  backGroundMean backGroundCI  ...
                  backGroundPercDiffMean backGroundPercDiffCI ...
                  trendMean trendCI ...
                  trendR2Mean trendR2CI ...
                  trendPredMean trendPredCI ...
                  trendPredPercDiffMean trendPredPercDiffCI ...
                  trendPredAbsDiffMean trendPredAbsDiffCI];
              
    varNamesPrint = {'2020 mean' ...
                '2019 mean' ...
                '2020 % difference from 2019' ...
                'Background start year' ...
                'Background mean' ...
                '2020 difference from background' ...
                'trend (yr^{-1})' ...
                'trend R2' ...
                '2020 predicted from trend' ...
                '2020 % difference from trend' ...
                '2020 abs difference from trend'};
    
    
    statsTablePrintMeanRow = {
                  [num2str(lockDownMean,precString)] ...
                  [num2str(prevYearMean,precString)] ...
                  [num2str(prevYearPercDiffMean,precString) ] ...
                  [num2str(backGroundStartYear)] ...
                  [num2str(backGroundMean,precString)] ...
                  [num2str(backGroundPercDiffMean,precString) ] ...
                  [num2str(trendMean,precString) ] ...
                  [num2str(trendR2Mean,'%.2f') ] ...
                  [num2str(trendPredMean,precString) ] ...
                  [num2str(trendPredPercDiffMean,precString)] ...
                  [num2str(trendPredAbsDiffMean,precString) ]};              
    
    statsTablePrintLCLRow = {
                  ['(' num2str(lockDownCI(1),precString) ', ' ] ...
                  ['(' num2str(prevYearCI(1),precString) ', ' ] ...
                  ['(' num2str(prevYearPercDiffCI(1),precString) ', ' ] ...
                  [''] ...
                  ['(' num2str(backGroundCI(1),precString) ', ' ] ...
                  ['(' num2str(backGroundPercDiffCI(1),precString) ', ' ] ...
                  ['(' num2str(trendCI(1),precString) ', ' ] ...
                  ['(' num2str(trendR2CI(1),'%.2f') ', ' ] ...
                  ['(' num2str(trendPredCI(1),precString) ', ' ] ...
                  ['(' num2str(trendPredPercDiffCI(1),precString) ', ' ] ...
                  ['(' num2str(trendPredAbsDiffCI(1),precString) ', ' ]};   
              
    statsTablePrintUCLRow = {
                  [num2str(lockDownCI(2),precString) ')'] ...
                  [num2str(prevYearCI(2),precString) ')'] ...
                  [num2str(prevYearPercDiffCI(2),precString)  ')'] ...
                  ['' ]...
                  [num2str(backGroundCI(2),precString)   ')'] ...
                  [num2str(backGroundPercDiffCI(2),precString)  ')'] ...
                  [num2str(trendCI(2),precString)  ')'] ...
                  [num2str(trendR2CI(2),'%.2f')  ')'] ...
                  [num2str(trendPredCI(2),precString)  ')'] ...
                  [num2str(trendPredPercDiffCI(2),precString) ')'] ...
                  [num2str(trendPredAbsDiffCI(2),precString) ')']};              

	statsTablePrint=[statsTablePrintMeanRow;statsTablePrintLCLRow;statsTablePrintUCLRow];
        
end

function tiledImage = AddImageToPanel(currFN,tiledImage,currRowStart,currColStart,nTileRows,nTileCols,leftCropFac,rightCropFac,topCropFac,botCropFac);
    [A,map] = imread(currFN);
    nRowNative = size(A,1);
    nColNative = size(A,2);
    leftCrop = floor(nColNative*leftCropFac);
    rightCrop = floor(nColNative*rightCropFac);
    topCrop = floor(nRowNative*topCropFac);
    botCrop = floor(nRowNative*botCropFac);

    if (isempty(tiledImage))
        tiledImage = 0*ones(nTileRows*(nRowNative-topCrop-botCrop),nTileCols*(nColNative-leftCrop-rightCrop),3);
        tiledImage = uint8(tiledImage);
    end
    
    croppedImage = A(topCrop:(nRowNative-botCrop-1),leftCrop:(nColNative-rightCrop+-1),:);
    %currRowStart = nTileRows-ceil((currPanelI)/nTileCols)+1;
    %currColStart = mod(currDateI-1,nTileCols)+1;

    pixStartRow = (currRowStart-1)*(nRowNative-topCrop-botCrop)+1;
    pixEndRow = pixStartRow+nRowNative-topCrop-botCrop-1;
    pixStartCol = (currColStart-1)*(nColNative-leftCrop-rightCrop)+1;
    pixEndCol = pixStartCol+nColNative-leftCrop-rightCrop-1;

    tiledImage(pixStartRow:pixEndRow,pixStartCol:pixEndCol,:) = croppedImage;
end
                


function MakeTrendAndDiffPlot(currStatsTable,regionName,currLetter,currVarName,currSatName,currUnits,precString,statName,...
    xMargin,yMargin,xGap,yGap,axWidth,axHeight,panelIndexCol,panelIndexRow,figH,dateRangeStr,logTransform,percDiff)
%
% this makes the plots of how trends and 2020 departures depend on the
% start of the background period (Fig 6,8 in paper)
%
%{
% stats 
  statsTable = [backGroundStartYear ... 1
                  lockDownMean lockDownCI ...2,3,4
                  prevYearMean prevYearCI ...5,6,7
                  prevYearPercDiffMean prevYearPercDiffCI ...8,9,10
                  backGroundMean backGroundCI  ...11,12,13
                  backGroundPercDiffMean backGroundPercDiffCI ...14,15,16
                  trendMean trendCI ...17,18,19
                  trendR2Mean trendR2CI ...20,21,22
                  trendPredMean trendPredCI ...23,24,25
                  trendPredPercDiffMean trendPredPercDiffCI ...26,27,28
                  trendPredAbsDiffMean trendPredAbsDiffCI]; 29,30,31
%}
    
	set(0, 'currentfigure', figH);
    currSubAx = axes();
    
    nYears = size(currStatsTable,1);
    %close all;
    %figure('visible','on')
    
    hold on;

    if (logTransform)
        yLabel = [currSatName ' ln(' currVarName ')' ];
    else
        yLabel = [currSatName ' ' currVarName ];
    end
    
    yyaxis left;
    hold on;
    currColor = 'r';
    for currYearI = 1:nYears
        backgroundYears(currYearI,1) = currStatsTable(currYearI,1);
        currY(currYearI,1) = currStatsTable(currYearI,17);
        currYLCL(currYearI,1) = currStatsTable(currYearI,18);
        currYUCL(currYearI,1) = currStatsTable(currYearI,19);
    end
    line(backgroundYears,currY,'Color',currColor,'LineWidth',1.5);
    hold on;
    x2 = [backgroundYears', fliplr(backgroundYears')];
    inBetween = [currYLCL', fliplr(currYUCL')];
    fillH = fill(x2, inBetween,currColor);%'FaceColor',currColor,'EdgeColor',currColor);
    set(fillH,'facealpha',.2,'edgealpha',0);
    
    ax = gca;
    ax.YAxis(1).Color = currColor;
    yLabel = [currSatName ' ' currVarName ];
    if (~strcmp(currUnits,''))
       yLabel = ['Trend in ' yLabel ' (' currUnits ' yr^{-1})']; 
    else
        yLabel = ['Trend in ' yLabel ' (yr^{-1})']; 
    end
    ylabel(yLabel);
    currYLim = get(ax,'YLim');
    % make Ylimit span 0;
    if (currYLim(1)*currYLim(2) > 0)
        if (currYLim(1) >0)
            currYLim = [-currYLim(1)/2 currYLim(2)];
        elseif (currYLim(2) <0)
            currYLim = [currYLim(1) -currYLim(2)/2];
        end
    end

    % pad a little bit just in case (like if original YLim had a 0
    yWidth = currYLim(2)-currYLim(1);
    set(ax,'YLim',[currYLim(1)-yWidth/10 currYLim(2)+yWidth/10]);
    currYLim = get(ax,'YLim');    
    % now shift to top
    set(ax,'YLim',[currYLim(1)-yWidth currYLim(2)]);
    line([min(backgroundYears)-.5 max(backgroundYears)],[0 0],'color',currColor,'LineStyle','--');

    
    yyaxis right;
    hold on;
    currColor = 'm';
    
    % % difference
    if (percDiff)
        for currYearI = 1:nYears
            backgroundYears(currYearI,1) = currStatsTable(currYearI,1);
            currY(currYearI,1) = currStatsTable(currYearI,26);
            currYLCL(currYearI,1) = currStatsTable(currYearI,27);
            currYUCL(currYearI,1) = currStatsTable(currYearI,28);
        end
    else    
    
        % absolute difference
        for currYearI = 1:nYears
            backgroundYears(currYearI,1) = currStatsTable(currYearI,1);
            currY(currYearI,1) = currStatsTable(currYearI,29);
            currYLCL(currYearI,1) = currStatsTable(currYearI,30);
            currYUCL(currYearI,1) = currStatsTable(currYearI,31);
        end
    end
    line(backgroundYears,currY,'Color',currColor,'LineWidth',1.5);
    hold on;
    x2 = [backgroundYears', fliplr(backgroundYears')];
    inBetween = [currYLCL', fliplr(currYUCL')];
    fillH = fill(x2, inBetween,currColor);%'FaceColor',currColor,'EdgeColor',currColor);
    set(fillH,'facealpha',.2,'edgealpha',0);
    ax = gca;
    ax.YAxis(2).Color = currColor;
    yLabel = [currSatName ' ' currVarName ];
    if (percDiff)
        yLabel = ['% difference between 2020 and prediction, ' yLabel]; 
    else
        if (~strcmp(currUnits,''))
           yLabel = ['Difference between 2020 and prediction, ' yLabel ' (' currUnits ')']; 
        else
            yLabel = ['Difference between 2020 and prediction, ' yLabel]; 
        end
    end
    ylabel(yLabel);                
    currYLim = get(ax,'YLim');
    % make Ylimit span 0;
    if (currYLim(1)*currYLim(2) > 0)
        if (currYLim(1) >0)
            currYLim = [-currYLim(1)/2 currYLim(2)];
        elseif (currYLim(2) <0)
            currYLim = [currYLim(1) -currYLim(2)/2];
        end
    end
    
    % pad a little bit just in case (like if original YLim had a 0
    yWidth = currYLim(2)-currYLim(1);
    set(ax,'YLim',[currYLim(1)-yWidth/10 currYLim(2)+yWidth/10]);
    currYLim = get(ax,'YLim');    
    set(ax,'YLim',[currYLim(1) currYLim(2)+yWidth]);
    line([min(backgroundYears) max(backgroundYears)+.5],[0 0],'color',currColor,'LineStyle','--');

    xlabel('Detrending start year');
    xlim([min(backgroundYears)-.5 max(backgroundYears)+.5]);

    titleVarName = [currLetter currSatName ' ' currVarName ' ' regionName ', ' dateRangeStr];
    title(titleVarName);
    
	set(currSubAx,'Color',[1 1 1]);% box off doesn't work,'box','off')

    set(currSubAx,'Units','inches');                
    currXPos = (xMargin +  (xGap+axWidth)*(panelIndexCol - 1)) ;
    currYPos = (yMargin + (yGap+axHeight)*(panelIndexRow -1 )) ;
    newAxPos = [currXPos currYPos axWidth axHeight];
    set(currSubAx,'Position',newAxPos);        
    

end
